﻿--[[

Configuration Screen Functions

]]

function EFM_CS_DisableSlider(slider)
	local name = slider:GetName();
	getglobal(name.."Thumb"):Hide();
	getglobal(name.."Title"):SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	getglobal(name.."Low"):SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	getglobal(name.."High"):SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
end

function EFM_CS_EnableSlider(slider)
	local name = slider:GetName();
	getglobal(name.."Thumb"):Show();
	getglobal(name.."Title"):SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
	getglobal(name.."Low"):SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
	getglobal(name.."High"):SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
end

-- Function: Config Screen init routine.
function EFM_CS_Init()
	EFM_CS_Slider_TimerLocSliderHigh:SetText("200");
	EFM_CS_Slider_TimerLocSliderLow:SetText("-200");
	EFM_CS_Slider_TimerLocSlider:SetValueStep(10);

        EFM_CS_Slider_TimerSizeSliderHigh:SetText("2.0");
        EFM_CS_Slider_TimerSizeSliderLow:SetText("0.1");
        EFM_CS_Slider_TimerSizeSlider:SetValueStep("0.05");

	local myButtonText	= getglobal("EFM_ConfigScreen_DoneText");
	myButtonText:SetFont(GameFontNormal:GetFont(), 12);

	local myButtonText	= getglobal("EFM_ConfigScreen_DefaultsText");
	myButtonText:SetFont(GameFontNormal:GetFont(), 12);

	local myButtonText	= getglobal("EFM_CS_Button_LoadHordeText");
	myButtonText:SetFont(GameFontNormal:GetFont(), 12);

	local myButtonText	= getglobal("EFM_CS_Button_LoadAllianceText");
	myButtonText:SetFont(GameFontNormal:GetFont(), 12);
end

-- Function: Routine to update configuration options.
function EFM_CS_OnShow()
	-- Set the check buttons to their defaults.
	EFM_CS_Button_Timer:SetChecked(EFM_MyConf.Timer);
	EFM_CS_Button_ShowTimerBar:SetChecked(EFM_MyConf.ShowTimerBar);
	EFM_CS_Button_ShrinkStatusBar:SetChecked(EFM_MyConf.ShrinkStatusBar);
	EFM_CS_Button_ZoneMarker:SetChecked(EFM_MyConf.ZoneMarker);
	EFM_CS_Button_ContinentOverlay:SetChecked(EFM_MyConf.ContinentOverlay);
	EFM_CS_Button_DruidPaths:SetChecked(EFM_MyConf.DruidPaths);
	EFM_CS_Button_UpdateRecorded:SetChecked(EFM_MyConf.LoadAll);
	EFM_CS_Button_LoadAll:SetChecked(EFM_MyConf.LoadAll);

	-- Set the slider details.
	EFM_CS_Slider_TimerLocSliderTitle:SetText(format(EFM_GUITEXT_DisplaySlider, EFM_MyConf.TimerPosition));
	EFM_CS_Slider_TimerLocSlider:SetValue(EFM_MyConf.TimerPosition);

        EFM_CS_Slider_TimerSizeSliderTitle:SetText(format(EFM_GUITEXT_SizeSlider, EFM_MyConf.TimerSize));
        EFM_CS_Slider_TimerSizeSlider:SetValue(EFM_MyConf.TimerSize);

	-- Set the Timer checkbox data.
	if (EFM_MyConf.Timer == false) then
		EFM_CS_Button_ShowTimerBar:Disable();
		EFM_CS_Button_ShowTimerBarText:SetTextColor(0.5, 0.5, 0.5);
		EFM_CS_Button_ShrinkStatusBar:Disable();
		EFM_CS_Button_ShrinkStatusBarText:SetTextColor(0.5, 0.5, 0.5);
		EFM_CS_DisableSlider(EFM_CS_Slider_TimerLocSlider);
		EFM_CS_DisableSlider(EFM_CS_Slider_TimerSizeSlider);
	else
		EFM_CS_Button_ShowTimerBar:Enable();
		EFM_CS_Button_ShowTimerBarText:SetTextColor(1, 0.82, 0);
		EFM_CS_Button_ShrinkStatusBar:Enable();
		EFM_CS_Button_ShrinkStatusBarText:SetTextColor(1, 0.82, 0);
		EFM_CS_EnableSlider(EFM_CS_Slider_TimerLocSlider);
		EFM_CS_EnableSlider(EFM_CS_Slider_TimerSizeSlider);
	end
end

-- Function: Set the program defaults.
function EFM_CS_SetDefaults()
	EFM_MyConf.Timer		= true;
	EFM_MyConf.ZoneMarker		= true;
	EFM_MyConf.DruidPaths		= false;
	EFM_MyConf.ShowTimerBar		= true;
	EFM_MyConf.ShrinkStatusBar	= true;
	EFM_MyConf.ShowLargeBar		= false;
	EFM_MyConf.TimerPosition	= -150;
	EFM_MyConf.TimerSize            = 1.0;
	EFM_MyConf.UpdateRecorded	= false;
	EFM_MyConf.LoadAll		= true;
	EFM_MyConf.ContinentOverlay	= true;
	EFM_CS_OnShow();
end

-- Function: Routine to handle position slider.
function EFM_CS_Slider_Changed()
	if (EFM_MyConf == nil) then
		return;
	end

	if (EFM_MyConf.Timer == false) then
		return;
	end

	local newValue = EFM_CS_Slider_TimerLocSlider:GetValue();

	EFM_MyConf.TimerPosition = newValue;
	EFM_CS_Slider_TimerLocSliderTitle:SetText(format(EFM_GUITEXT_DisplaySlider, newValue));

	-- New for Size
	EFM_MyConf.TimerSize = EFM_CS_Slider_TimerSizeSlider:GetValue();
	EFM_CS_Slider_TimerSizeSliderTitle:SetText(format(EFM_GUITEXT_SizeSlider, EFM_MyConf.TimerSize));
end

