-- Some day I'll be a real header with informative description!


CalendarFu = LibStub("AceAddon-3.0"):NewAddon("CalendarFu", "AceConsole-3.0", "LibFuBarPlugin-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CalendarFu", true)

-- Debugging stuff
local isDebug = false
function CalendarFu:DebugPrint(debugString)
	if not isDebug then return end
	CalendarFu:Print(debugString)
end

-- TODO: make colors configurable!
local COLOR_EVENT_MOD = {
	CREATOR = "|cff9999ff", -- light blue: creator
	MODERATOR = "|cffbbbbbb", -- light gray: moderator
}
local COLOR_EVENT_INV = {
	"|cffff0000", -- red: invited
	"|cff00ff00", -- green: accepted
	"|cff44bbbb", -- teal: declined
	"|cff00cc00", -- dark green: confirmed
	"|cffcc0000", -- dark red: out
	"|cff0000ff", -- blue: standby
}

local eventInfo = {}

local db

local options = {
  type = "group",
  name = "FuBar_CalendarFu",
  handler = CalendarFu,
	get = function(info) return db[ info[#info] ] end,
	set = function(info, value) db[ info[#info] ] = value end,
  args = {
	  General = {
	  	type = "group",
	  	order = 1,
	  	name = L["General"],
	  	desc = L["General"],
	  	args = {
	  		Description = {
	  			order = 1,
	  			type = "description",
	  			name = L["CalendarFu is a FuBar plugin for Blizzard Calendar"],
	  		}, -- Description
	  		DaysForward = {
	  			type = "range",
	  			order = 2,
	  			name = L["Days Forward"],
	  			desc = L["Defines how many forward to scan for events"],
	  			min = 0,
	  			max = 90,
	  			step = 1,
	  			isPercent = false,
	  			get = function(info) return db.General.DaysForward end,
	  			set = function(info, v)
	  				db.General.DaysForward = v
	  				--CalendarFu:OnUpdateFuBarTooltip()
	  				CalendarFu:OnUpdateFuBarText()
	  			end,
	  		},
				EventTypes = {
					type = "multiselect",
					order = 3,
					name = L["Event Types"],
					desc = L["Event Types"],
					values = {
						BLIZZARD = L["Blizzard"],
						RAID = L["Raid"],
						DUNGEON = L["Dungeon"],
						PVP = L["Pvp"],
						MEETING = L["Meeting"],
						OTHER = L["Other"],
					},
					get = function(info, k) return db.General.EventTypes[k] end,
					set = function(info, k, v)
						db.General.EventTypes[k] = v
						CalendarFu:OnUpdateFuBarText()
						CalendarFu:OnUpdateFuBarTooltip()
					end,
				}, -- EventTypes
				CalendarTypes = {
					type = "multiselect",
					order = 4,
					name = L["Calendar Types"],
					desc = L["Calendar Types"],
					values = {
						PLAYER = L["Player"],
						GUILD = L["Guild"],
						ARENA = L["Arena"],
						HOLIDAY = L["Holiday"],
					},
					get = function(info, k) return db.General.CalendarTypes[k] end,
					set = function(info, k, v)
						db.General.CalendarTypes[k] = v
						CalendarFu:OnUpdateFuBarText()
						CalendarFu:OnUpdateFuBarTooltip()
					end,
				}, -- CalendarTypes
				SequenceTypes = {
					type = "multiselect",
					order = 5,
					name = L["Sequence Types"],
					desc = L["Sequence Types"],
					values = {
						ONGOING = L["Ongoing"],
						STARTSEQ = L["Start"],
						ENDSEQ = L["End"],
						NONESEQ = L["No Sequence"],
					},
					get = function(info, k) return db.General.SequenceTypes[k] end,
					set = function(info, k, v)
						db.General.SequenceTypes[k] = v
						CalendarFu:OnUpdateFuBarText()
						CalendarFu:OnUpdateFuBarTooltip()
					end,
				}, -- SequenceTypes
				NotifyTypes = {
					type = "multiselect",
					order = 6,
					name = L["Notify Types"],
					desc = L["Types of Notifications to Show in CalendarFu Title Text"],
					values = {
						INVITED = L["INVITED"],
						ACCEPTED = L["ACCEPTED"],
						DECLINED = L["DECLINED"],
						CONFIRMED = L["CONFIRMED"],
						OUT = L["OUT"],
						STANDBY = L["STANDBY"],
					},
					get = function(info, k) return db.General.NotifyTypes[k] end,
					set = function(info, k, v)
						db.General.NotifyTypes[k] = v
						CalendarFu:OnUpdateFuBarText()
						CalendarFu:OnUpdateFuBarTooltip()
					end,
				}, -- NotifyTypes
				ColorSettings = {
					type = "group",
					order = 7,
					name = L["Color Settings"],
					desc = L["Color Settings"],
					guiInline = true,
					args = {
						DelimiterColor = {
							type = "color",
							order = 8,
							name = L["Event Delimiter Color"],
							desc = L["Color of the brackets, parenthesis etc."],
							hasAlpha = true,
							get = function(info)
								local t = db.General.DelimiterColor
								return t.r, t.g, t.b, t.a
							end,
							set = function(info, r, g, b, a)
								local t = db.General.DelimiterColor
								t.r, t.g, t.b, t.a = r, g, b, a
								CalendarFu:OnUpdateFuBarText()
								CalendarFu:OnUpdateFuBarTooltip()
							end,
						}, -- DelimiterColor
						DateColor = {
							type = "color",
							order = 9,
							name = L["Event Date Color"],
							desc = L["Event Date Color"],
							hasAlpha = true,
							get = function(info)
								local t = db.General.DateColor
								return t.r, t.g, t.b, t.a
							end,
							set = function(info, r, g, b, a)
								local t = db.General.DateColor
								t.r, t.g, t.b, t.a = r, g, b, a
								CalendarFu:OnUpdateFuBarText()
								CalendarFu:OnUpdateFuBarTooltip()
							end,
						}, -- DateColor
						TimeColor = {
							type = "color",
							order = 10,
							name = L["Event Time Color"],
							desc = L["Event Time Color"],
							hasAlpha = true,
							get = function(info)
								local t = db.General.TimeColor
								return t.r, t.g, t.b, t.a
							end,
							set = function(info, r, g, b, a)
								local t = db.General.TimeColor
								t.r, t.g, t.b, t.a = r, g, b, a
								CalendarFu:OnUpdateFuBarText()
								CalendarFu:OnUpdateFuBarTooltip()
							end,
						}, -- TimeColor
						TitleColor = {
							type = "color",
							order = 11,
							name = L["Event Title Color"],
							desc = L["Event Title Color"],
							hasAlpha = true,
							get = function(info)
								local t = db.General.TitleColor
								return t.r, t.g, t.b, t.a
							end,
							set = function(info, r, g, b, a)
								local t = db.General.TitleColor
								t.r, t.g, t.b, t.a = r, g, b, a
								CalendarFu:OnUpdateFuBarText()
								CalendarFu:OnUpdateFuBarTooltip()
							end,
						}, -- TitleColor
					}, -- args of ColorSettings
				}, -- ColorSettings
			}, -- args of General
		}, -- General
		FuBar = {
			order = -4,
			type = "group",
			name = L["FuBar Options"],
			desc = L["FuBar Options"],
			args = {
				showIcon = {
					type = "toggle",
					order = 3,
					name = L["Show Icon"],
					desc = L["Show or Hide CalendarFu FuBar Icon"],
					get = function(info) return CalendarFu:IsFuBarIconShown() end,
					set = function(info, v) CalendarFu:ToggleFuBarIconShown() end,
				},
				showText = {
					type = "toggle",
					order = 4,
					name = L["Show Text"],
					desc = L["Show or Hide CalendarFu FuBar Text"],
					get = function(info) return CalendarFu:IsFuBarTextShown() end,
					set = function(info, v) CalendarFu:ToggleFuBarTextShown() end,
				},
				position = {
					type = "select",
					order = 5,
					name = L["Position"],
					desc = L["Position"],
					values = {LEFT = L["Left"], CENTER = L["Center"], RIGHT = L["Right"]},
					get = function() return CalendarFu:GetPanel() and CalendarFu:GetPanel():GetPluginSide(CalendarFu) end,
					set = function(info, val)
						if CalendarFu:GetPanel() and CalendarFu:GetPanel().SetPluginSide then
							CalendarFu:GetPanel():SetPluginSide(CalendarFu, val)
						end
					end,
				}
			}
		}, -- FuBar
	},
}

local defaults = {
	profile = {
		General = {
			DaysForward = 3,
			EventTypes = {
				BLIZZARD = true,
				RAID = true,
				DUNGEON = true,
				PVP = true,
				MEETING = false,
				OTHER = false,
			},
			CalendarTypes = {
				PLAYER = true,
				GUILD = true,
				ARENA = false,
				HOLIDAY = false,
			},
			SequenceTypes = {
				ONGOING = false,
				STARTSEQ = true,
				ENDSEQ = true,
				NONESEQ = true,
			},
			NotifyTypes = {
				INVITED = true,
				ACCEPTED = false,
				DECLINED = false,
				CONFIRMED = false,
				OUT = false,
				STANDBY = false,
			},
			DelimiterColor = { r = 1.0, g = 1.0, b = 0.0, a = 1.0 },
			DateColor = { r = 0.7, g = 0.7, b = 0.7, a = 1.0 },
			TimeColor = { r = 1.0, g = 1.0, b = 1.0, a = 1.0 },
			TitleColor = { r = 1.0, g = 1.0, b = 0.0, a = 1.0 },
		},
		FuBar = {
			showIcon = true,
			showText = true,
			position = RIGHT,
		},
	},
}

CalendarFu.Options = options;

local CalendarFu_InviteStatus = {
	L["INVITED"],
	L["ACCEPTED"],
	L["DECLINED"],
	L["CONFIRMED"],
	L["OUT"],
	L["STANDBY"],	
}

function CalendarFu:MapEventType(eventType)
	if eventType == 0 then return "BLIZZARD" end
	if eventType == 1 then return "RAID" end
	if eventType == 2 then return "DUNGEON" end
	if eventType == 3 then return "PVP" end
	if eventType == 4 then return "MEETING" end
	if eventType == 5 then return "OTHER" end
	return -1
end

function CalendarFu:MapCalendarType(calendarType)
	if calendarType == "PLAYER" then return "PLAYER" end
	if calendarType == "GUILD" then return "GUILD" end
	if calendarType == "ARENA" then return "ARENA" end
	if calendarType == "HOLIDAY" then return "HOLIDAY" end
	return -1
end

function CalendarFu:MapSequenceType(sequenceType)
	if sequenceType == "ONGOING" then return "ONGOING" end
	if sequenceType == "START" then return "STARTSEQ" end
	if sequenceType == "END" then return "ENDSEQ" end
	if sequenceType == "" then return "NONESEQ" end
	return -1
end

function CalendarFu:MapInviteStatus(inviteStatus)
	if inviteStatus == 1 then return "INVITED" end
	if inviteStatus == 2 then return "ACCEPTED" end
	if inviteStatus == 3 then return "DECLINED" end
	if inviteStatus == 4 then return "CONFIRMED" end
	if inviteStatus == 5 then return "OUT" end
	if inviteStatus == 6 then return "STANDBY" end
end

function CalendarFu:OnInitialize()
	self:InitConfig()
end

function CalendarFu:OnEnable()
end

function CalendarFu:OnDisable()
end

function CalendarFu:OnFuBarClick()
	if Calendar_Toggle then
		Calendar_Toggle()
		local numEvents = self:UpdateEventInfo()
	end
end

function CalendarFu:OnUpdateFuBarTooltip()
	
	GameTooltip:AddLine("|cffffff00" .. L["Click|r to Show or Hide Blizzard Calendar"])
	GameTooltip:AddLine("|cffffff00" .. L["Right-click|r opens the Configuration"])
	
	local numEvents = self:UpdateEventInfo()
	
	self:DebugPrint("numEvents = " .. numEvents)
	
	if numEvents > 0 then
		GameTooltip:AddLine(" ")
	else
		return
	end

	-- get first event date for reference
	local eventDate = ""

	-- Add events to Tooltip
	for eventIndex = 1, numEvents, 1 do

		local eventAddLineLeft = "  "
		local eventAddLineRight = ""

		if eventDate ~= eventInfo[eventIndex]["date"] then
			eventDate = eventInfo[eventIndex]["date"]
			GameTooltip:AddLine(self:TextColorFormat(db.General.DateColor) .. eventDate .. "|r")
		end
		
		-- Left
		eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. "[|r"
		if eventInfo[eventIndex]["seqtype"] == "ONGOING" then
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.TimeColor) .. L["ONGOING"] .. "|r"
		else
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.TimeColor) .. eventInfo[eventIndex]["time"] .. "|r"
		end
		
		eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. "]|r "
		eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.TitleColor) .. eventInfo[eventIndex]["title"] .. "|r "
		if eventInfo[eventIndex]["modstatus"] == "CREATOR" then
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. "(|r"
			eventAddLineLeft = eventAddLineLeft .. COLOR_EVENT_MOD["CREATOR"] .. L["creator"] .. "|r"
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. ")|r"
		end
		
		if eventInfo[eventIndex]["modstatus"] == "MODERATOR" then
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. "(|r"
			eventAddLineLeft = eventAddLineLeft .. COLOR_EVENT_MOD["MODERATOR"] .. L["moderator"] .. "|r"
			eventAddLineLeft = eventAddLineLeft .. self:TextColorFormat(db.General.DelimiterColor) .. ")|r"
		end
			
		-- Right
		eventAddLineRight = eventAddLineRight .. COLOR_EVENT_INV[eventInfo[eventIndex]["invstatus"]] ..
			CalendarFu_InviteStatus[eventInfo[eventIndex]["invstatus"]] .. "|r"
		GameTooltip:AddDoubleLine(eventAddLineLeft, eventAddLineRight)

	end
	
end

function CalendarFu:OnUpdateFuBarText()
	
	local numEvents = self:UpdateEventInfo()

	local fubarTitleText = ""
	fubarTitleText = fubarTitleText .. L["FuBar Title"]
	
	if #eventInfo <= 0 then 
		self:SetFuBarText(L["FuBar Title"])
		return
	end
	
	local inviteInfo = { 0, 0, 0, 0, 0, 0 }
	
	local eventIndex = 1
	
	for eventIndex = 1, #eventInfo, 1 do
		self:DebugPrint("inviteInfo @ " .. eventInfo[eventIndex]["invstatus"] .. " = " .. inviteInfo[eventInfo[eventIndex]["invstatus"]])
		inviteInfo[eventInfo[eventIndex]["invstatus"]] = inviteInfo[eventInfo[eventIndex]["invstatus"]] + 1
	end
	
	if #inviteInfo <= 0 then
		self:SetFuBarText(L["FuBar Title"])
		return
	end	
	
	fubarTitleText = fubarTitleText .. " " .. self:TextColorFormat(db.General.DelimiterColor) .. "(|r"
	
	local inviteInfoIndex = 1
	local notifyCount = 0
	
	for inviteInfoIndex = 1, #inviteInfo, 1 do
		local notifyTypeKey = self:MapInviteStatus(inviteInfoIndex)
		if inviteInfo[inviteInfoIndex] > 0 and db.General.NotifyTypes[notifyTypeKey] == true then
			fubarTitleText = fubarTitleText .. COLOR_EVENT_INV[inviteInfoIndex] .. inviteInfo[inviteInfoIndex] .. "|r"
			notifyCount = notifyCount + 1
		end
	end
	
	fubarTitleText = fubarTitleText .. self:TextColorFormat(db.General.DelimiterColor) .. ")|r"
	
	if notifyCount > 0 then
		self:SetFuBarText(fubarTitleText)
	else
		self:SetFuBarText(L["FuBar Title"])
	end
	
end

function CalendarFu:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
end

function CalendarFu:InitConfig()
	
	-- Ace3 stuff
  self.db = LibStub("AceDB-3.0"):New("CalendarFuDB", defaults)
  db = self.db.profile
  LibStub("AceConfig-3.0"):RegisterOptionsTable("CalendarFu", options, nil)
	local acd3 = LibStub("AceConfigDialog-3.0")

	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	self.optionsFrames = {}
	self.optionsFrames.CalendarFu = acd3:AddToBlizOptions("CalendarFu", nil, nil, "General")
	self.optionsFrames["FuBar"] = acd3:AddToBlizOptions("CalendarFu", L["FuBar Options"], "CalendarFu", "FuBar")
	self.OpenMenu = self.ShowConfig

  -- Fubar3 stuff
	self:SetFuBarOption('hasIcon', true)
	self:SetFuBarOption("iconPath", [[Interface\AddOns\FuBar_CalendarFu\icon]])
	self:SetFuBarOption('defaultPosition', "RIGHT")
	self:SetFuBarOption('hideWithoutStandby', true)
	self:SetFuBarOption('configType', "AceConfig-3.0")
	self:SetFuBarOption("tooltipType", "GameTooltip")
	self:SetFuBarText(L["FuBar Title"]);
	
end

function CalendarFu:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames["FuBar"])
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.CalendarFu)
end

function CalendarFu:OnUpdate()
	self:OnUpdateFuBarText()
end

-- Update eventInfo data
function CalendarFu:UpdateEventInfo()
	
	self:DebugPrint("UpdateEventInfo ENTER")
	
	if not IsAddOnLoaded("Blizzard_Calendar") then
		local loaded, reason = LoadAddOn("Blizzard_Calendar")
		if not loaded then
			if reason ~= nil then
				self:DebugPrint("Could not load Blizzard_Calendar (" .. reason .. ")")
			end
			return 0
		else
			self:DebugPrint("Blizzard_Calendar loaded OK")
			-- HACK HACK
			-- For some reason even if Blizzard_Calendar is loaded ok
			-- API calls fail if we don't open the frame at least once
			-- So we open and close it
			if Calendar_Show then
				Calendar_Show()
			end
			if Calendar_Hide then
				Calendar_Hide()
			end
		end
	end
	
	eventInfo = {}
	
	local monthOffset = 0
	local _, month, day, year = CalendarGetDate();
	if day <= 0 or day > 31 or month <= 0 or month > 12 then
		return 0
	end
	
	-- Loop for daysForward number of times
	for dayOffset = 0, db.General.DaysForward, 1 do
		
		self:DebugPrint("Processing day " .. dayOffset .. " out of " .. db.General.DaysForward)
		
		-- Build a list of events
		local numEvents = CalendarGetNumDayEvents(monthOffset, day+dayOffset)
		
		if numEvents > 0 then
		
			self:DebugPrint("numEvents = " .. numEvents)	
			
			local eventIndex = 1
			
			for eventIndex = 1, numEvents, 1 do
				local title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus = CalendarGetDayEvent(monthOffset, day+dayOffset, eventIndex)
				if title and hour and minute and calendarType and sequenceType and eventType and texture and modStatus and inviteStatus then
					-- Add selected events to Tooltip
					self:DebugPrint("eventType = " .. eventType)
					local eventTypeIndex = CalendarFu:MapEventType(eventType)
					self:DebugPrint("eventTypeIndex = " .. eventTypeIndex)
					if eventTypeIndex == -1 then break end
					local eventTypeSelected = db.General.EventTypes[eventTypeIndex]
					if eventTypeSelected == true then
						self:DebugPrint("eventTypeSelected = true")
					else
						self:DebugPrint("eventTypeSelected = false")
					end
					self:DebugPrint("calendarType = " .. calendarType)
					local calendarTypeIndex = CalendarFu:MapCalendarType(calendarType)
					self:DebugPrint("calendarTypeIndex = " .. calendarTypeIndex)
					if calendarTypeIndex == -1 then break end
					local calendarTypeSelected = db.General.CalendarTypes[calendarTypeIndex]
					if calendarTypeSelected == true then
						self:DebugPrint("calendarTypeSelected = true")
					else
						self:DebugPrint("calendarTypeSelected = false")
					end
					self:DebugPrint("sequenceType = " .. sequenceType)
					local sequenceTypeIndex = CalendarFu:MapSequenceType(sequenceType)
					self:DebugPrint("sequenceTypeIndex = " .. sequenceTypeIndex)
					if sequenceTypeIndex == -1 then break end
					local sequenceTypeSelected = db.General.SequenceTypes[sequenceTypeIndex]
					if sequenceTypeSelected == true then
						self:DebugPrint("sequenceTypeSelected = true")
					else
						self:DebugPrint("sequenceTypeSelected = false")
					end
					if eventTypeSelected == true and calendarTypeSelected == true and sequenceTypeSelected == true then
						local nextIndex = #eventInfo + 1
						eventInfo[nextIndex] = {}
						eventInfo[nextIndex]["evtype"] = eventType
						eventInfo[nextIndex]["caltype"] = calendarType
						eventInfo[nextIndex]["seqtype"] = sequenceType
						eventInfo[nextIndex]["modstatus"] = modStatus
						eventInfo[nextIndex]["invstatus"] = inviteStatus
						eventInfo[nextIndex]["time"] = string.format("%02d:%02d", hour, minute)
						eventInfo[nextIndex]["date"] = string.format("%02d.%02d.%04d", day+dayOffset, month, year)
						eventInfo[nextIndex]["title"] = title
					end
				end
			end
		end
	end
	
	self:DebugPrint("UpdateEventInfo EXIT")

	return #eventInfo
	
end

local function RGBAToHex(r, g, b, a)
	-- WoW r,g,b,a scale is 0.0-1.0 so multiply first
	r = r * 255
	g = g * 255
	b = b * 255
	a = a * 255
	return string.format("%02x%02x%02x%02x", a, r, g, b)
end

function CalendarFu:TextColorFormat(c)
	if c ~= nil then
		return "|c" .. RGBAToHex(c.r, c.g, c.b, c.a)
	else
		return "|c00000000"
	end
end