setfenv(1, LuckyCharms.fenv)

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local AUTOHIDE_DELAY_TIME = 1

local function Button_OnClick(self, button, down)
	if not skipClick then
		if groupLeader then
			PutRaidIconOnTarget('target', self.id)
		elseif groupMarking then
			SendSyncData("M", self.id, syncOwner)
		end
	end
	skipClick = nil
end

local isMoving, oldX, oldY
local function Button_OnDragStart(self, button)
	isMoving, oldX, oldY = self:GetFrameStrata(), self:GetCenter()
	self:ClearAllPoints()
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', oldX, oldY)
	self:SetFrameStrata('TOOLTIP')
	self:StartMoving()
end

local function Button_OnDragStop(self, button)
	if not isMoving then
		return
	end
	self:StopMovingOrSizing()
	self:SetFrameStrata(isMoving)
	isMoving = nil
	local old, x, y, adj, iMax = self.killOrder, self:GetCenter()
	if x < frame:GetLeft() + buttonSize * (old - 0.5) + profile.spacing * (old - 1) then
		adj, iMax = -1, 1
	else
		adj, iMax = 1, NUM_ICONS
	end
	local new = iMax
	for i = old, iMax - adj, adj do
	    if adj * (buttons[killOrder[i + adj]]:GetCenter() - x) > 0 then
	        new = i
	        break
	    end
	end
    if AdjustKillOrder(self.id, new) and groupLeader then
		SendSyncData("C", format("%u%u", self.id, new))
	end
	return abs(oldX - x) > profile.sensitivity or abs(oldY - y) > profile.sensitivity
end

local function Button_OnMouseDown(self, button)
	GameTooltip:Hide()
	skipLeave = true
	if button == 'LeftButton' and profile.unlockIcons and not (IsSyncRestricted() or IsModifierKeyDown()) then
		if Dewdrop:IsOpen(frame) then
			Dewdrop:Close()
		end
		Button_OnDragStart(self, button)
		return
	end
	return Frame_OnMouseDown(frame, button)
end

local function Button_OnMouseUp(self, button)
	if button == 'LeftButton' and Button_OnDragStop(self, button) then
		skipClick, skipLeave = true, true
		return
	end
	return Frame_OnMouseUp(frame, button)
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
function Button_OnEnter(self, motion)
	addon:CancelScheduledEvent(title .. "_OnLeave")
	if not self.id then
		return
	end
    raidIconMenu = self.id
	if Dewdrop:IsOpen(frame) then
	    Dewdrop:Close()
	    addon:OpenMenu()                                                        -- Do it this way to prevent menu from closing early
	    return
	end
	GameTooltip:SetOwner(frame, 'ANCHOR_BOTTOMLEFT', 0, GetButtonMaxSize() * -3 / 11)
	GameTooltip:SetText(iconName[self.id], 1, 1, 1)
	if groupLeader then
		if GetBindingKey('RAIDTARGET' .. iconSuffix[self.id]) then
			GameTooltip:AppendText(format(" %s(%s)%s", NORMAL_FONT_COLOR_CODE, GetBindingKey('RAIDTARGET' .. iconSuffix[self.id]), FONT_COLOR_CODE_CLOSE))
		end
	    if raidIconMenu > 0 then
			GameTooltip:AddLine("Place the " .. strlower(iconName[self.id]) .. " icon on your target.", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
		else
		    GameTooltip:AddLine("Clear any existing icon from your target.", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine("Limits broadcasts to only the icons to the left.", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
		end
	end
	GameTooltip:Show()
end

function Button_OnLeave(self, motion)
	addon:CancelScheduledEvent(title .. "_OnLeave")
	if GameTooltip:IsOwned(frame) then
		GameTooltip:Hide()
	end
	if Dewdrop:IsOpen(frame) or skipLeave then
		addon:ScheduleEvent(title .. "_OnLeave", Button_OnLeave, AUTOHIDE_DELAY_TIME)
		skipLeave = nil
	end
end

--[[----------------------------------------------------------------------------
Initialize module
------------------------------------------------------------------------------]]
frame:SetScript('OnEnter', Button_OnEnter)
frame:SetScript('OnLeave', Button_OnLeave)
for i = 0, NUM_RAID_ICONS do
	buttons[i] = CreateFrame('Button', nil, frame)
	buttons[i].id = i
	buttons[i].texture = buttons[i]:CreateTexture(nil, 'BACKGROUND')
	buttons[i].texture:SetAllPoints(buttons[i])
	if i ~= 0 then
		buttons[i].texture:SetTexture([[Interface\TargetingFrame\UI-RaidTargetingIcon_]] .. i)
	else
		buttons[i].texture:SetTexture([[Interface\Addons\FuBar_LuckyCharms\None]])
	end
	buttons[i]:SetScript('OnMouseDown', Button_OnMouseDown)
	buttons[i]:SetScript('OnMouseUp', Button_OnMouseUp)
	buttons[i]:SetScript('OnEnter', Button_OnEnter)
	buttons[i]:SetScript('OnLeave', Button_OnLeave)
	buttons[i]:SetScript('OnClick', Button_OnClick)
	buttons[i]:EnableMouse(true)
	buttons[i]:SetMovable(true)
	buttons[i]:Show()
end
