LuckyCharms = AceLibrary('AceAddon-2.0'):new('FuBarPlugin-2.1', 'AceDB-2.0', 'AceEvent-2.0')

LuckyCharms.fenv = setmetatable({ }, { __index = _G })
setfenv(1, LuckyCharms.fenv)

addon = LuckyCharms
addon.frame = addon:CreateBasicPluginFrame("LuckyCharms_Frame")
addon.title = "LuckyCharms"

buttons, frame, profile, title = { }, addon.frame, false, addon.title
iconName = { [0] = "None", [1] = "Star", [2] = "Circle", [3] = "Diamond", [4] = "Triangle", [5] = "Moon", [6] = "Square", [7] = "Cross", [8] = "Skull" }
iconSuffix = { [0] = "NONE", [1] = "1", [2] = "2", [3] = "3", [4] = "4", [5] = "5", [6] = "6", [7] = "7", [8] = "8" }
killOrderDefault = { 0, 1, 2, 3, 4, 5, 6, 7, 8 }

addon:RegisterDB("LuckyCharmsDB")
addon:RegisterDefaults('profile', {
	version       = "",															-- Don't change
	modifier      = 0,
	noPartySync   = false,
	noRaidSync    = false,
	partyMarking  = false,
	pullCounter   = true,
	pullCount     = 10,
	readyCheck    = true,
	scaling       = 1,
	sensitivity   = 2,
	size          = 12,
	sizeOption    = "Automatic",
	spacing       = 3,
	unlockIcons	  = false })

MIN_BUTTON_SIZE           = 5
MAX_SENSITIVITY           = floor(MIN_BUTTON_SIZE / 2)
NUM_RAID_ICONS            = 8
NUM_ICONS				  = NUM_RAID_ICONS + 1

buttonSize, maxButtonSize = MIN_BUTTON_SIZE

--[[----------------------------------------------------------------------------
Helper functions
------------------------------------------------------------------------------]]
function NilFunction()
end

function printf(...)
	DEFAULT_CHAT_FRAME:AddMessage(format(tostringall(...)))
end

function strproper(text)
	text = text or ""
	return strupper(strsub(text, 1, 1)) .. strlower(strsub(text, 2))
end

function table_clear(t)
	if type(t) ~= 'table' then
	    return
	end
	for i, v in pairs(t) do
		table_clear(v)
	    t[i] = nil
	end
	t[1] = true
	t[1] = nil
end

function table_copy(t)
	if type(t) ~= 'table' then
	    return t
	end
	local r = { }
	for i, v in pairs(t) do
		r[i] = table_copy(v)
	end
	return r
end

function xor(a, b)
	return (a and not b) or (not a and b)
end

--[[----------------------------------------------------------------------------
Internal functions
------------------------------------------------------------------------------]]
function AdjustKillOrder(icon, new)
	local old = buttons[icon].killOrder
	local adj = old > new and -1 or 1
	for i = old, new - adj, adj do
   		killOrder[i] = killOrder[i + adj]
	end
	killOrder[new] = icon
	CopyKillOrderToButtons()
	RefreshDisplay()
	return new ~= old
end

function AssignGroupMemberIcon(member)
	if groupLeader and groupAssign[raidIconMenu] ~= member then
		groupAssign[raidIconMenu] = member
		SendSyncData("A", raidIconMenu .. member)
	end
end

function Broadcast(showAssignments, showKillOrder)
	if not groupLeader then
		return
	end
	local text, headerSent = ""
	for i = 1, NUM_ICONS do
		if killOrder[i] == 0 then
			break
		end
	    text = format("%s {%s} ", text, iconName[killOrder[i]])
		if showAssignments and groupAssign[killOrder[i]] then
			if not headerSent then
				SendChatMessage("Assignments:", grouped)
				headerSent = true
			end
			SendChatMessage(format("{%s} = %s", iconName[killOrder[i]], groupAssign[killOrder[i]]), grouped)
		end
	end
	if showKillOrder and strlen(text) > 0 then
		SendChatMessage("Kill order: " .. text, 'RAID_WARNING')
	end
end

function ClearAssignment(icon)
	if not groupLeader then
		return
	end
	icon = icon or raidIconMenu
	if icon >= 1 and icon <= NUM_RAID_ICONS then
		groupAssign[icon] = nil
	end
   	SendSyncData("A", icon)
end

function CopyKillOrderToButtons()
	for i = 1, NUM_ICONS do
		buttons[killOrder[i]].killOrder = i
	end
end

function GetButtonSize()
	return 32
end

function GetButtonMaxSize()
	return GetButtonSize() * 2
end

function IsSyncRestricted()
	if groupLeader or syncDisabled then
		return
	end
	return syncMode
end

function PullCountDown(abort)
	if (pullCount and pullCount <= 0) or abort then
	    if abort then
	    	SendChatMessage("STOP", 'RAID_WARNING')
	    	SendChatMessage("Pull Aborted", 'RAID_WARNING')
	    	SendChatMessage("Stand Down", 'RAID_WARNING')
		else
			SendChatMessage("Pull.  Kill.  Profit.", 'RAID_WARNING')
		end
		addon:CancelScheduledEvent(title .. "_CountDownTimer")
		pullCount = nil
		if Dewdrop:IsOpen(frame) then
		    Dewdrop:Refresh()
		end
		return
	end
	if not pullCount then
		addon:ScheduleRepeatingEvent(title .. "_CountDownTimer", PullCountDown, 1)
		pullCount = profile.pullCount
	    SendChatMessage(format("Pulling in %u seconds...", pullCount), 'RAID_WARNING')
	else
		if pullCount > 5 then
			SendChatMessage(format("Pulling in %u seconds...", pullCount), grouped)
		else
		    SendChatMessage(format("Pulling in %u second%s...", pullCount, pullCount == 1 and "" or "s"), 'RAID_WARNING')
		end
	end
   	pullCount = pullCount - 1
end

function PutRaidIconOnTarget(target, icon)
	if UnitExists(target) and icon ~= GetRaidTargetIndex(target) then
		SetRaidTarget(target, icon)
	end
end

function RefreshButtonSize()
	buttonSize, maxButtonSize = GetButtonSize(), GetButtonMaxSize()
	if profile.sizeOption == "Automatic" then
		buttonSize = sqrt(maxButtonSize * maxButtonSize * 0.75)
	elseif profile.sizeOption == "Fixed" then
	    buttonSize = profile.size
	elseif profile.sizeOption == "Normal" then
	    buttonSize = buttonSize + profile.modifier
	else
	    buttonSize = sqrt(maxButtonSize * maxButtonSize * profile.scaling)
	end
	buttonSize = max(MIN_BUTTON_SIZE, min(buttonSize, maxButtonSize))
	RefreshDisplay()
end

function RefreshDisplay()
	for i = 0, NUM_RAID_ICONS do
		buttons[i]:SetHeight(buttonSize)
		buttons[i]:SetWidth(buttonSize)
		buttons[killOrder[i + 1]]:ClearAllPoints()
   		buttons[killOrder[i + 1]]:SetPoint('LEFT', frame, 'LEFT', buttonSize * i + profile.spacing * i, 0)
	end
	frame:SetHeight(maxButtonSize)
	frame:SetWidth(buttonSize * NUM_ICONS + profile.spacing * NUM_RAID_ICONS)
end

local oldGrouped
function UpdateStatus(force)
	grouped, groupLeader, groupMarking, maxGroupMembers, syncDisabled = nil, nil, nil, 0, nil
	if GetNumRaidMembers() > 0 then
		grouped = 'RAID'
		groupLeader = IsRaidLeader() or IsRaidOfficer()
		maxGroupMembers = 40
		syncDisabled = profile.noRaidSync
	elseif GetNumPartyMembers() > 0 then
		grouped = 'PARTY'
		groupLeader = UnitIsPartyLeader('player')
		groupMarking = profile.partyMarking
		maxGroupMembers = 4
		syncDisabled = profile.noPartySync
	end
   	UpdateSyncMonitor()
	if grouped ~= oldGrouped or force then
		oldGrouped = grouped
		BuildGroupRoster()
	end
end

function ValueCompare(a, b)
	local compare = a - b
	return compare ~= 0 and compare or nil
end

function VersionCompare(a, b)
	local a_major, a_minor, a_revision = VersionSplit(a)
	local b_major, b_minor, b_revision = VersionSplit(b)
	return ValueCompare(a_major, b_major) or ValueCompare(a_minor, b_minor) or ValueCompare(a_revision, b_revision)
end

function VersionSplit(version)
	local major, minor, revision = strsplit(".", version)
	return tonumber(major) or 0, tonumber(minor) or 0, tonumber(revision) or 0
end

--[[----------------------------------------------------------------------------
Global functions
------------------------------------------------------------------------------]]
function addon:OnEnable()
	addon:OnProfileEnable()
    addon:RegisterEvent('CHAT_MSG_ADDON', RcvdSyncData)
	addon:RegisterBucketEvent('PARTY_MEMBERS_CHANGED', 1, UpdateGroupRoster)	-- + 'RAID_ROSTER_UPDATE'
end

function addon:OnInitialize()
	Frame_OnMouseDown = frame:GetScript('OnMouseDown')							-- Must do this here since FuBar hooks after FuBarPlugin
	Frame_OnMouseUp = frame:GetScript('OnMouseUp')
end

function addon:OnProfileEnable()
	profile = addon.db.profile
	if type(profile.killOrder) ~= 'table' or #profile.killOrder ~= #killOrderDefault then
		profile.killOrder = table_copy(killOrderDefault)
	end
	if VersionCompare(profile.version, addon.version) then						-- Handle any issues with different version's saved variables
		if profile.sensitivity > MAX_SENSITIVITY then
			profile.sensitivity = MAX_SENSITIVITY
		end
		profile.version = addon.version
	end
	syncVersion = addon.version
	killOrder = profile.killOrder
	CopyKillOrderToButtons()
	RefreshButtonSize()
	UpdateStatus()
end
