setfenv(1, LuckyCharms.fenv)

Dewdrop = AceLibrary('Dewdrop-2.0')

GROUP_MENU_ENTRY	   = 5
GROUP_MENU_ENTRY_ORDER = GROUP_MENU_ENTRY + 1
MAX_BUTTON_MODIFIER	   = 25
MAX_BUTTON_SIZE		   = 50
MAX_BUTTON_SPACING	   = 25
MIN_BUTTON_MODIFIER	   = -25

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local function GetValue(k)
	return profile[k]
end

local function SetValue(k, v)
	profile[k] = v
end

local function SetValueWithRefresh(k, v)
	profile[k] = v
	RefreshButtonSize()
end

local function SetValueWithStatusUpdate(k, v)
	profile[k] = v
	UpdateStatus()
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
settingsMenu = { name = "Options", type = 'group', desc = format("%s's options.", title), args = {

	{ name = "Appearance", type = 'group', order = 1, desc = format("Options to change the size and look of %s in the FuBar panel.", title), args = {

		{ name = "Size Options", type = 'text', order = 1, passValue = "sizeOption",
		  desc = format("Choose how the size of the buttons will be determined.  The size cannot exceed the bounds of the FuBar panel or be smaller than %s x %s.", MIN_BUTTON_SIZE, MIN_BUTTON_SIZE),
		   set = SetValueWithRefresh,
		   get = GetValue,
  	validate = { "Automatic", "Fixed", "Normal", "Scale" },
 	validateDesc = { ["Automatic"] = "Size is determined automatically based on FuBar's settings.",
				   	 ["Fixed"] = "Size is set to a fixed value.",
				   	 ["Normal"] = "Size will match FuBar's font size and may be modified further by a fixed amount.",
				   	 ["Scale"] = "Size will be scaled relative to the maximum as determined by FuBar's settings." } },

		{ name = "Set Size", type = 'range', order = 2, passValue = "size", max = MAX_BUTTON_SIZE, min = MIN_BUTTON_SIZE, step = 1,
		  desc = "Set a value to use for the size of the buttons.",
   		hidden = function() return profile.sizeOption ~= "Fixed" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Set Modifier", type = 'range', order = 3, passValue = "modifier", max = MAX_BUTTON_MODIFIER, min = MIN_BUTTON_MODIFIER, step = 1,
		  desc = "Set the amount to modify FuBar's font size by.",
	   	hidden = function() return profile.sizeOption ~= "Normal" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Set Scaling", type = 'range', order = 4, passValue = "scaling", max = 1, min = 0, step = 0.01, isPercent = true,
		  desc = "Set the amount to scale the size relative to the maximum.",
		hidden = function() return profile.sizeOption ~= "Scale" end,
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ name = "Spacing", type = 'range', order = 5, passValue = "spacing", max = MAX_BUTTON_SPACING, min = 0, step = 1,
		  desc = "Set the spacing between the buttons.",
		   set = SetValueWithRefresh,
		   get = GetValue },

		{ type = 'header', order = 6 },

		{ name = "Sensitivity", type = 'range', order = 7, passValue = "sensitivity", max = MAX_SENSITIVITY, min = 0, step = 1,
	  	  desc = "Set the amount you can move an unlocked button and still have it count as a click.",
		   set = SetValue,
		   get = GetValue },

		{ name = "Unlock", type = 'toggle', order = 8, passValue = "unlockIcons",
	  	  desc = "Unlock the buttons to change their order.  The order the buttons are in (from left to right) is the kill order.",
	  disabled = IsSyncRestricted,
		   set = SetValue,
		   get = GetValue } } },

	{ name = "Extras", type = 'group', order = 2, desc = "Options for when leading a party/raid.", args = {

		{ name = "Enable Pull Counter", type = 'toggle', order = 1, passValue = "pullCounter",
	  	  desc = "Show the pull counter option.",
		   set = SetValue,
		   get = GetValue },

		{ name = "Enable Ready Check", type = 'toggle', order = 2, passValue = "readyCheck",
	  	  desc = "Show the ready check option.",
		   set = SetValue,
		   get = GetValue },

		{ name = "Pull Count", type = 'range', order = 3, passValue = "pullCount", max = 15, min = 3, step = 1,
	  	  desc = "Set the number to count down from for the pull counter.",
		   set = SetValue,
		   get = GetValue } } },

	{ name = "Sync Mode", type = 'group', order = 4, desc = "Options for syncing data.", args = {

		{ name = "Disable in Party", type = 'toggle', order = 1, passValue = "noPartySync",
	  	  desc = "Don't sync data when in a party.",
		   set = SetValueWithStatusUpdate,
		   get = GetValue },

		{ name = "Disable in Raid", type = 'toggle', order = 2, passValue = "noRaidSync",
	  	  desc = "Don't sync data when in a raid.",
		   set = SetValueWithStatusUpdate,
		   get = GetValue },

		{ name = "Party Marking", type = 'toggle', order = 3, passValue = "partyMarking",
	  	  desc = "Allow anyone in the party syncing with LuckyCharms to be able to mark targets.  This option must also be selected by the party leader to work.",
	  disabled = function() return profile.noPartySync end,
		   set = SetValueWithRefresh,
		   get = GetValue } } } } }

--------------------------------------------------------------------------------

optionsMenu = { type = 'group', args = {

	{ name = "Broadcast", type = 'group', order = 1, desc = "Options for relaying information to the party/raid.", hidden = function() return not groupLeader end, args = {

		{ name = "Assignments", type = 'execute', order = 1,
		  desc = "Send the list of assignments (if any) to party/raid chat.",
    	  func = function() Broadcast(true) end },

		{ name = "Kill Order", type = 'execute', order = 2,
		  desc = "Send the kill order as a raid warning.",
    	  func = function() Broadcast(nil, true) end } } },

	{ name = "Pull Counter", type = 'group', order = 2, desc = "Options for starting/stopping a pull count down.", hidden = function() return not (groupLeader and profile.pullCounter) end, args = {

		{ name = "Begin", type = 'execute', order = 1,
		  desc = "Start a pull count down.",
	  disabled = function() return pullCount end,
	      func = function() PullCountDown() end },

		{ name = "Cancel", type = 'execute', order = 2,
		  desc = "Stop the pull count down.",
	  disabled = function() return not pullCount end,
	      func = function() PullCountDown(true) end } } },

	{ name = "Ready Check", type = 'execute', order = 3,
	  desc = "Send a ready check to the party/raid.",
	hidden = function() return not (groupLeader and profile.readyCheck) end,
	  func = function() DoReadyCheck() end },

	{ type = 'header', order = 4 },

--------------------------------------------------------------------------------

	{ type = 'header', order = GROUP_MENU_ENTRY, iconHeight = 16, iconWidth = 16 },

	{ name = "None", type = 'execute', order = GROUP_MENU_ENTRY + 2, iconHeight = 10, iconWidth = 10, icon = [[Interface\Addons\FuBar_LuckyCharms\None]],
	  desc = "Clear this assignment.",
	hidden = function() return not (groupLeader and groupAssign[raidIconMenu]) end,
	  func = ClearAssignment },

	{ type = 'header', order = GROUP_MENU_ENTRY + 3 }, settingsMenu } }
		
--------------------------------------------------------------------------------

function RefreshMenu()
	if Dewdrop:IsOpen(frame) then
		Dewdrop:Refresh()
	end
end

--[[----------------------------------------------------------------------------
Global
------------------------------------------------------------------------------]]
local memberWithIcon
function addon:OnMenuRequest()
	if grouped and raidIconMenu > 0 then
		local menuIcon
		for i = 1, 2 do                                                         -- Clear old icon on first pass then add new icon
		    if memberWithIcon and memberClass[memberWithIcon] then
				if grouped == 'RAID' then
				    groupMenu[memberClass[memberWithIcon]].icon = menuIcon
				    groupMenu[memberClass[memberWithIcon]].args[memberWithIcon].icon = menuIcon
				else
				    groupMenu[memberWithIcon].icon = menuIcon
				end
		    end
            memberWithIcon = groupAssign[raidIconMenu]
			menuIcon = [[Interface\TargetingFrame\UI-RaidTargetingIcon_]] .. raidIconMenu
		end
	    optionsMenu.args[GROUP_MENU_ENTRY].name = ("%s%s%s"):format(groupLeader and "Assign " or "", groupLeader and strlower(iconName[raidIconMenu]) or strproper(iconName[raidIconMenu]), groupLeader and " to:" or "")
	    optionsMenu.args[GROUP_MENU_ENTRY].icon = menuIcon
	    Dewdrop:FeedAceOptionsTable(optionsMenu)
	else
	    Dewdrop:FeedAceOptionsTable(settingsMenu)
	end
end

--[[----------------------------------------------------------------------------
Initialize module
------------------------------------------------------------------------------]]
Dewdrop:InjectAceOptionsTable(addon, settingsMenu)
raidIconMenu = 0
