setfenv(1, LuckyCharms.fenv)

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local activeMember, validUnit, rosterType = { }, { }

local function DisableGroupMenuEntry(member)
	return not (activeMember[member] and UnitIsConnected(activeMember[member]))
end

local function CreateClassMenuEntry(class)
	return { name = class, type = 'group', order = GROUP_MENU_ENTRY_ORDER, desc = class, iconHeight = 10, iconWidth = 10, args = { } }
end

local function CreateGroupMenuEntry(member, class)
	return { name = member, type = 'execute', order = GROUP_MENU_ENTRY_ORDER, desc = class, disabled = DisableGroupMenuEntry, func = AssignGroupMemberIcon, passValue = member, iconHeight = 10, iconWidth = 10 }
end

local function AddMemberToRoster(unitID)
	local member = rosterType and UnitExists(unitID) and UnitName(unitID)
	if not member or member == 'Unknown' then return end
	local class = UnitClass(unitID)
	local classKnown = class and class ~= 'Unknown'
	activeMember[member] = (memberClass[member] or classKnown) and unitID
	if memberClass[member] or not classKnown then return end
	if rosterType == "RAID" then
		if not groupMenu[class] then
			groupMenu[class] = CreateClassMenuEntry(class)
			groupMenu[class].count = 0
		end
        groupMenu[class].args[member] = CreateGroupMenuEntry(member, class)
		groupMenu[class].count = groupMenu[class].count + 1
	else
        groupMenu[member] = CreateGroupMenuEntry(member, class)
	end
	memberClass[member] = class
end

local function RemoveMemberFromRoster(member, skipClearAssign)
	if not (rosterType and memberClass[member]) then return end
	local class = memberClass[member]
	if rosterType == "RAID" then
		table_clear(groupMenu[class].args[member])
		groupMenu[class].args[member] = nil
		groupMenu[class].count = groupMenu[class].count - 1
		if groupMenu[class].count <= 0 then
			table_clear(groupMenu[class])
			groupMenu[class] = nil
		end
	else
		table_clear(groupMenu[member])
		groupMenu[member] = nil
	end
	activeMember[member] = nil
	memberClass[member] = nil
	if not skipClearAssign then
		for i = 1, NUM_RAID_ICONS do
			if groupAssign[i] == member then
				groupAssign[i] = nil
			end
		end
	end
end

local function ClearGroupRoster()
	for member in pairs(memberClass) do
		RemoveMemberFromRoster(member, 1)
	end
	table_clear(validUnit)
	rosterType = nil
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
groupAssign, groupMenu, memberClass = { }, optionsMenu.args, { }

function BuildGroupRoster()
	ClearGroupRoster()
	if grouped then
		rosterType = grouped
		if grouped == 'PARTY' then
			AddMemberToRoster('player')
			validUnit['player'] = true
		end
		local unitType, unitID = strlower(grouped)
		for i = 1, maxGroupMembers do
			unitID = unitType .. i
			AddMemberToRoster(unitID)
			validUnit[unitID] = true
		end
		for i = 1, NUM_RAID_ICONS do
		    if groupAssign[i] and not memberClass[groupAssign[i]] then
		        groupAssign[i] = nil
		    end
		end
	else
	    table_clear(groupAssign)
	end
	RefreshMenu()
end

function GetUnitID(name)
	local UnitID
	for unitID in pairs(validUnit) do
		if name == UnitName(unitID) then
			UnitID = unitID
			break
		end
    end
    return UnitID
end

function UpdateGroupRoster()
	UpdateStatus()
	table_clear(activeMember)
	if not rosterType then return end
	for unitID in pairs(validUnit) do
		AddMemberToRoster(unitID)
	end
	for member in pairs(memberClass) do
		if not activeMember[member] then
			RemoveMemberFromRoster(member)
		end
	end
	RefreshMenu()
end
