setfenv(1, LuckyCharms.fenv)

SYNC_PREFIX = ""

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local DETECT_TIME	   = 1
local MONITOR_INTERVAL = 2
local PULSE_INTERVAL   = 3
local TIME_OUT		   = 4

local _killOrder, monitorTime, syncMT, default, sync = strrep("%s", NUM_ICONS), GetTime(), { __index = function() return NilFunction end }, { }, { }
default[DETECT_TIME]	  = 18
default[MONITOR_INTERVAL] = 1
default[PULSE_INTERVAL]	  = 15
default[TIME_OUT] 		  = 20

local ProcessSyncCommand, ProcessSyncRequest = setmetatable({ }, syncMT), setmetatable({ }, syncMT)

local function DelayedSyncStart()
	if syncPending then
		SendSyncData("S")
		syncPending = nil
	end
end

local function LoadDefaultTimings()
	for i, v in pairs(default) do
		sync[i] = v
	end	
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
playerName = UnitName('player')

function RcvdSyncData(prefix, data, msgType, sender)
	if prefix ~= SYNC_PREFIX or syncDisabled then
	    return
	end
	local cmd = strsub(data, 1, 1)
	data = strsub(data, 2)
	if msgType == grouped then
		monitorTime = GetTime()
		if not syncMode then
			syncMode = true
			syncOwner = syncOwner or sender
			syncPending = nil
			SendSyncData("R", "AKV", sender)
		end
		if cmd == "S" and sender ~= syncOwner then								-- Pick owner if conflict
			local deltaStrLen = strlen(sender) - strlen(syncOwner or "")
			if not syncOwner or deltaStrLen < 0 or (deltaStrLen == 0 and sender < syncOwner) then
			    syncOwner = sender
		    end
		end
	end
	if not syncMode or sender == playerName then
		return
	end
	ProcessSyncCommand[cmd](data, sender)
end

function SendSyncData(cmd, data, who)
	local msgType = (who and 'WHISPER') or ((groupLeader or cmd == "F") and grouped)
	if syncDisabled or who == playerName or not msgType then
		return
	end
   	SendAddonMessage(SYNC_PREFIX, cmd .. (data or ""), msgType, who)
end

function StartSyncMode()
	if not (syncMode or syncPending) then
		syncOwner = nil
		addon:ScheduleEvent(title .. "_SyncPending", DelayedSyncStart, sync[DETECT_TIME])
		syncPending = true
	end
end

function StopSyncMode()
	syncMode = nil
	syncPending = nil
	syncOwner = nil
end

function SyncModeMonitor()
	if syncMode then
        local deltaTime = GetTime() - monitorTime
		if not grouped then
			StopSyncMode()
		elseif syncOwner == playerName then
			if not groupLeader then
				syncOwner = SendSyncData("F")
			elseif deltaTime >= sync[PULSE_INTERVAL] then
	    		SendSyncData("S")
			end
		elseif deltaTime > sync[TIME_OUT] then
	        StopSyncMode()
		end
	elseif grouped and groupLeader then
		StartSyncMode()
	elseif syncPending and not groupLeader then
		StopSyncMode()
	end
end

function UpdateSyncMonitor()
	if not syncDisabled and not syncMonitor then
   		addon:ScheduleRepeatingEvent(title .. "_SyncMonitor", SyncModeMonitor, sync[MONITOR_INTERVAL])
   		syncMonitor = true
	elseif syncDisabled and syncMonitor then
	    addon:CancelScheduledEvent(title .. "_SyncMonitor")
	    syncMonitor = nil
	    StopSyncMode()
	end
end

--[[----------------------------------------------------------------------------
Sync command functions
------------------------------------------------------------------------------]]
ProcessSyncCommand.A = function(data)											-- Assign an icon to someone
	groupAssign[tonumber(strsub(data, 1, 1))] = strlen(data) > 1 and strsub(data, 2) or nil
	RefreshMenu()
end

ProcessSyncCommand.C = function(data)											-- Change kill order
    AdjustKillOrder(tonumber(strsub(data, 1, 1)), tonumber(strsub(data, 2, 2)))
end

ProcessSyncCommand.F = function()												-- Forfeit ownership
    syncOwner = groupLeader and SendSyncData("S") or nil
end

ProcessSyncCommand.I = function(data)											-- Information
	printf("LuckyCharms: %s", data)
end

ProcessSyncCommand.K = function(data)											-- Kill order to use
    for i = 1, NUM_ICONS do
   	    killOrder[i] = tonumber(strsub(data, i, i))
   	end
   	CopyKillOrderToButtons()
	RefreshDisplay()
end

ProcessSyncCommand.M = function(data, sender)									-- Mark sender's target with an icon
	local unitID = GetUnitID(sender)
	if unitID then
	    PutRaidIconOnTarget(unitID .. 'target', tonumber(strsub(data, 1, 1)))
	end
end

ProcessSyncCommand.R = function(data, sender)									-- Request data
	local cmd
	while data ~= "" do
		cmd = strsub(data, 1, 1)
		data = strsub(data, 2)
		ProcessSyncRequest[cmd](sender)
	end
end

ProcessSyncCommand.T = function(data)											-- Timing data
	sync[1], sync[2], sync[3], sync[4] = strsplit(",", data)
	for i = 1, 4 do
		sync[i] = tonumber(sync[i])
		if not sync[i] then
			LoadDefaultTimings()
			SendSyncData("R", "T", syncOwner)
			return
		end
	end
end

ProcessSyncCommand.V = function(data, sender)									-- Version check
	local versionCheck = VersionCompare(data, syncVersion) or 0
	if versionCheck < 0 then
   		SendSyncData("V", syncVersion, sender)
	elseif versionCheck > 0 then
       	syncVersion = data
       	printf("LuckyCharms: A newer version (%s) has been detected.", syncVersion)
		if syncOwner == playerName then
			SendSyncData("V", syncVersion)
		end
		SendSyncData("R", "T", sender)
	end
end

--[[----------------------------------------------------------------------------
Sync request functions
------------------------------------------------------------------------------]]
ProcessSyncRequest.A = function(sender)											-- Request assignments
	for i = 1, NUM_RAID_ICONS do
		SendSyncData("A", i .. (groupAssign[i] or ""), sender)
	end
end

ProcessSyncRequest.K = function(sender)											-- Request kill order
	SendSyncData("K", _killOrder:format(unpack(killOrder)), sender)
end

ProcessSyncRequest.T = function(sender)											-- Request timing data
	SendSyncData("T", format("%s,%s,%s,%s", unpack(sync)), sender)
end

ProcessSyncRequest.V = function(sender)											-- Request version
	SendSyncData("V", syncVersion, sender)
end

--[[----------------------------------------------------------------------------
Initialize module
------------------------------------------------------------------------------]]
LoadDefaultTimings()