setfenv(1, MicroMenu.fenv)

--[[----------------------------------------------------------------------------
Local to module
------------------------------------------------------------------------------]]
local AUTOHIDE_DELAY_TIME, _SetChecked = 1

local function Button_OnClick(self, button, down)
	if not skipClick then
		self = menuData[self.id]
		if self.toggle then
			pcall(_G[self.toggle])
		else
			if not _G[self.frame] then
				pcall(_G[self.frame .. '_LoadUI'])
			end
			pcall(ToggleFrame, _G[self.frame])
		end
	end
	UpdateMicroButtons()
	skipClick = nil
end

local isMoving, oldX, oldY
local function Button_OnDragStart(self, button)
	isMoving, oldX, oldY = self:GetFrameStrata(), self:GetCenter()
	self:ClearAllPoints()
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', oldX, oldY)
	self:SetFrameStrata('TOOLTIP')
	self:StartMoving()
end

local function Button_OnDragStop(self, button)
	if not isMoving then
		return
	end
	self:StopMovingOrSizing()
	self:SetFrameStrata(isMoving)
	isMoving = nil
	local old, x, y, adj, iMax = self.order, self:GetCenter()
	if x < frame:GetLeft() + buttonSize * (old - 0.5) + profile.spacing * (old - 1) then
		adj, iMax = -1, 1
	else
		adj, iMax = 1, #menuOrder
	end
	local new = iMax
	for i = old, iMax - adj, adj do
	    if adj * (buttons[menuOrder[i + adj]]:GetCenter() - x) > 0 then
	        new = i
	        break
	    end
	end
    AdjustMenuOrder(self.id, new)
	return abs(oldX - x) > profile.sensitivity or abs(oldY - y) > profile.sensitivity
end

local function Button_OnMouseDown(self, button)
	GameTooltip:Hide()
	skipLeave = true
	if button == 'LeftButton' and profile.unlockIcons and not IsModifierKeyDown() then
		if Dewdrop:IsOpen(frame) then
			Dewdrop:Close()
		end
		Button_OnDragStart(self, button)
		return
	end
	return Frame_OnMouseDown(frame, button)
end

local function Button_OnMouseUp(self, button)
	if button == 'LeftButton' and Button_OnDragStop(self, button) then
		skipClick, skipLeave = true, true
		return
	end
	return Frame_OnMouseUp(frame, button)
end

local function SetTexCoord(frame, ...)
	local arg = select(1, ...)
	local coordType = type(arg)
	if coordType == 'function' then
		SetTexCoord(frame, arg())
	elseif coordType == 'table' then
		SetTexCoord(frame, unpack(arg))
	elseif coordType == 'nil' then
	    frame:SetTexCoord(0, 1, 0, 1)
	else
		frame:SetTexCoord(...)
	end
end

local function SetTexture(frame, ...)
	local arg = select(1, ...)
	local textureType = type(arg)
	if textureType == 'function' then
		SetTexture(frame, arg())
	elseif textureType == 'table' then
		if arg.texture then
			SetTexture(frame, arg.texture)
		    SetTexCoord(frame, arg.texCoord)
		else
		    SetTexture(frame, unpack(arg))
		end
	elseif textureType == 'boolean' or textureType == 'nil' then
	    frame:SetTexture(0, 0, 0, 1)
	else
		frame:SetTexture(...)
	end
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
function Button_OnEnter(self, motion)
	addon:CancelScheduledEvent(title .. "_OnLeave")
	if not self.id then
		return
	end
	if Dewdrop:IsOpen(frame) then
	    Dewdrop:Close()
	    addon:OpenMenu()                                                        -- Do it this way to prevent menu from closing early
	    return
	end
	self = menuData[self.id]
	GameTooltip:SetOwner(frame, 'ANCHOR_BOTTOMLEFT', 0, GetButtonMaxSize() * -3 / 11)
	GameTooltip:SetText(self.title or self.name, 1, 1, 1)
	if self.keyBind and GetBindingKey(self.keyBind) then
		GameTooltip:AppendText(format(" %s(%s)%s", NORMAL_FONT_COLOR_CODE, GetBindingKey(self.keyBind), FONT_COLOR_CODE_CLOSE))
	end
	if self.tooltip then
		GameTooltip:AddLine(self.tooltip, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
	end
	if self.tooltipExtend then
		self.tooltipExtend()
	end
	GameTooltip:Show()
end

function Button_OnLeave(self, motion)
	addon:CancelScheduledEvent(title .. "_OnLeave")
	if GameTooltip:IsOwned(frame) then
		GameTooltip:Hide()
	end
	if Dewdrop:IsOpen(frame) or skipLeave then
		addon:ScheduleEvent(title .. "_OnLeave", Button_OnLeave, AUTOHIDE_DELAY_TIME)
		skipLeave = nil
	end
end

--[[----------------------------------------------------------------------------
Initialize module
------------------------------------------------------------------------------]]
frame:SetScript('OnEnter', Button_OnEnter)
frame:SetScript('OnLeave', Button_OnLeave)

for k, v in pairs(menuData) do
	buttons[k] = CreateFrame('CheckButton', nil, frame)
	buttons[k].id = k
	buttons[k].texture = buttons[k]:CreateTexture(nil, 'ARTWORK')
	buttons[k].texture:SetAllPoints(buttons[k])
	buttons[k]:SetCheckedTexture([[Interface\Buttons\CheckButtonHilight]])
	buttons[k]:SetHighlightTexture([[Interface\Buttons\ButtonHilight-Square]])
	buttons[k]:SetPushedTexture([[Interface\Buttons\ButtonHilight-Square]])
	buttons[k]:GetPushedTexture():SetBlendMode('ADD')

	if v.setIcon then
		buttons[k]:SetScript('OnEvent', v.setIcon)
		buttons[k]:RegisterEvent('PLAYER_ENTERING_WORLD')
	elseif v.texture then
		SetTexture(buttons[k].texture, v.texture)
	elseif v.name then
		buttons[k].texture:SetTexture([[Interface\Buttons\UI-MicroButton]] .. '-' .. v.name .. '-Up')
	end

	SetTexCoord(buttons[k].texture, v.texCoord)

	if v.background then
		buttons[k].background = buttons[k]:CreateTexture(nil, 'BACKGROUND')
		buttons[k].background:SetAllPoints(buttons[k])
		SetTexture(buttons[k].background, v.background)
	end

	if v.overlay then
	    buttons[k].overlay = buttons[k]:CreateTexture(nil, 'OVERLAY')
		buttons[k].overlay:SetAllPoints(buttons[k])
		SetTexture(buttons[k].overlay, v.overlay)
	end

	if v.onUpdate then
	    buttons[k]:SetScript('OnUpdate', v.onUpdate)
	    buttons[k].updateInterval = 0
	end

	_SetChecked = _SetChecked or buttons[k].SetChecked
	buttons[k].SetChecked = NilFunction
	buttons[k]:SetButtonState('NORMAL', false)
	buttons[k]:SetScript('OnMouseDown', Button_OnMouseDown)
	buttons[k]:SetScript('OnMouseUp', Button_OnMouseUp)
	buttons[k]:SetScript('OnEnter', Button_OnEnter)
	buttons[k]:SetScript('OnLeave', Button_OnLeave)
	buttons[k]:SetScript('OnClick', Button_OnClick)
	buttons[k]:EnableMouse(true)
	buttons[k]:SetMovable(true)
	buttons[k]:Show()
end

--------------------------------------------------------------------------------

hooksecurefunc('HideUIPanel', UpdateMicroButtons)

hooksecurefunc('ShowUIPanel', function(frame)
	frame = frame:GetName()
	for _, v in pairs(menuOrder) do
		if frame == menuData[v].frame then
			_SetChecked(buttons[v], true)
	    elseif type(menuData[v].frames) == 'table' then
	    	for _, f in pairs(menuData[v].frames) do
	    		if frame == f then
	    			_SetChecked(buttons[v], true)
	    			break
	    		end
	    	end
		end
	end
end)

hooksecurefunc('UpdateMicroButtons', function()
	local checked
	for _, v in pairs(menuOrder) do
		checked = _G[menuData[v].frame] and _G[menuData[v].frame]:IsVisible()
		if not checked and type(menuData[v].frames) == 'table' then
		    for _, v in pairs(menuData[v].frames) do
				checked = _G[v] and _G[v]:IsVisible()
				if checked then break end
		    end
		end
		_SetChecked(buttons[v], checked)
	end
end)
