MicroMenu = AceLibrary('AceAddon-2.0'):new('FuBarPlugin-2.1', 'AceDB-2.0', 'AceEvent-2.0')

MicroMenu.fenv = setmetatable({ }, { __index = _G })
setfenv(1, MicroMenu.fenv)

addon = MicroMenu
addon.frame = addon:CreateBasicPluginFrame("MicroMenu_Frame")
addon.title = "MicroMenu"

buttons, frame, profile, title = { }, addon.frame, false, addon.title

addon:RegisterDB("MicroMenuDB")
addon:RegisterDefaults('profile', {
	version     = "",															-- Don't change
	latency		= 600,
	modifier    = 0,
	scaling     = 1,
	sensitivity = 2,
	size        = 12,
	sizeOption  = "Automatic",
	spacing     = 3,
	unlockIcons	= false })

MIN_BUTTON_SIZE           = 5
MAX_SENSITIVITY           = floor(MIN_BUTTON_SIZE / 2)

buttonSize, maxButtonSize = MIN_BUTTON_SIZE

--[[----------------------------------------------------------------------------
Helper functions
------------------------------------------------------------------------------]]
function NilFunction()
end

function get_index(name)
	local i
	for k, v in pairs(menuData) do
		if v.name == name then
			i = k
			break
		end
	end
	return i
end

function printf(...)
	DEFAULT_CHAT_FRAME:AddMessage(format(tostringall(...)))
end

function table_clear(t)
	if type(t) ~= 'table' then
	    return
	end
	for i, v in pairs(t) do
		table_clear(v)
	    t[i] = nil
	end
	t[1] = true
	t[1] = nil
end

function table_copy(t)
	if type(t) ~= 'table' then
	    return t
	end
	local r = { }
	for i, v in pairs(t) do
		r[i] = table_copy(v)
	end
	return r
end

function xor(a, b)
	return (a and not b) or (not a and b)
end

--[[----------------------------------------------------------------------------
Internal functions
------------------------------------------------------------------------------]]
function AdjustMenuOrder(id, new)
	local old = buttons[id].order
	local adj = old > new and -1 or 1
	for i = old, new - adj, adj do
   		menuOrder[i] = menuOrder[i + adj]
	end
	menuOrder[new] = id
	CopyMenuOrderToButtons()
	RefreshDisplay()
	return new ~= old
end

function CopyMenuOrderToButtons()
	for _, v in pairs(buttons) do
		v.order = nil
	end
	for k, v in pairs(menuOrder) do
		buttons[v].order = k
	end
end

function GetButtonSize()
	return 32
end

function GetButtonMaxSize()
	return GetButtonSize() * 2
end

function RefreshButtonSize()
	buttonSize, maxButtonSize = GetButtonSize(), GetButtonMaxSize()
	if profile.sizeOption == "Automatic" then
		buttonSize = sqrt(maxButtonSize * maxButtonSize * 0.75)
	elseif profile.sizeOption == "Fixed" then
	    buttonSize = profile.size
	elseif profile.sizeOption == "Normal" then
	    buttonSize = buttonSize + profile.modifier
	else
	    buttonSize = sqrt(maxButtonSize * maxButtonSize * profile.scaling)
	end
	buttonSize = max(MIN_BUTTON_SIZE, min(buttonSize, maxButtonSize))
	RefreshDisplay()
end

function RefreshDisplay()
	local offsetX
	for k, v in pairs(buttons) do
	    if v.order then
	        offsetX = v.order - 1
			v:SetHeight(buttonSize)
			v:SetWidth(buttonSize)
			v:ClearAllPoints()
			v:SetPoint('LEFT', frame, 'LEFT', buttonSize * offsetX + profile.spacing * offsetX, 0)
	        v:Show()
	    else
	        v:Hide()
	    end
	end
	frame:SetHeight(maxButtonSize)
	frame:SetWidth(buttonSize * #menuOrder + profile.spacing * (#menuOrder - 1))
end

function ValueCompare(a, b)
	local compare = a - b
	return compare ~= 0 and compare or nil
end

function VersionCompare(a, b)
	local a_major, a_minor, a_revision = VersionSplit(a)
	local b_major, b_minor, b_revision = VersionSplit(b)
	return ValueCompare(a_major, b_major) or ValueCompare(a_minor, b_minor) or ValueCompare(a_revision, b_revision)
end

function VersionSplit(version)
	local major, minor, revision = strsplit(".", version)
	return tonumber(major) or 0, tonumber(minor) or 0, tonumber(revision) or 0
end

--[[----------------------------------------------------------------------------
Global functions
------------------------------------------------------------------------------]]
function addon:OnEnable()
	addon:OnProfileEnable()
end

function addon:OnInitialize()
	Frame_OnMouseDown = frame:GetScript('OnMouseDown')							-- Must do this here since FuBar hooks after FuBarPlugin
	Frame_OnMouseUp = frame:GetScript('OnMouseUp')
end

function addon:OnProfileEnable()
	profile = addon.db.profile
	if type(profile.menuOrder) ~= 'table' or profile.menuCount ~= #menuOrderDefault then
		table_clear(profile.hidden)
		profile.menuCount = #menuOrderDefault
		profile.menuOrder = table_copy(menuOrderDefault)
	end
	if VersionCompare(profile.version, addon.version) then						-- Handle any issues with different version's saved variables
		if profile.sensitivity > MAX_SENSITIVITY then
			profile.sensitivity = MAX_SENSITIVITY
		end
		profile.version = addon.version
	end
	LATENCY_MID_RANGE = profile.latency / 2
	menuOrder = profile.menuOrder
	CopyMenuOrderToButtons()
	RefreshButtonSize()
end
