local quixote = LibStub("LibQuixote-2.0")
local LSM = LibStub("LibSharedMedia-3.0")

function Inquest:InitializeQuestLog()
	--Make quest log frame draggable
	QuestLogFrame:EnableMouse(true)
	QuestLogFrame:SetMovable(true)
	QuestLogFrame:SetClampedToScreen(true)
	self:HookScript(QuestLogFrame, "OnDragStart", "QuestLog_OnDragStart")
	self:HookScript(QuestLogFrame, "OnDragStop", "QuestLog_OnDragStop")
	
	--Set locked state of the quest log frame
	self:SetQuestLogLocked(self.db.profile.questLogLocked)
	
	self:RawHook("QuestLogTitleButton_OnClick", true)
	self:SecureHook("QuestLog_Update")
	
	--Reposition quest log when it is shown
	self:HookScript(QuestLogFrame, "OnShow", function() self:SetQuestLogPosition() end)
	
	quixote.RegisterCallback(self, "Objective_Update", "Objective_Update")
	
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_COMPLETE")
end

function Inquest:Objective_Update(event, questTitle, uid, desc, had, got, needed)
	local questComplete, _
	uid, _, _, _, _, _, questComplete = quixote:GetQuestByUid(uid)
	if(uid and not questComplete) then
		if(got >= needed and self.db.profile.objectiveCompleteSound) then
			PlaySoundFile(LSM:Fetch("sound", self.db.profile.objectiveCompleteSoundFile))
		elseif(got > had and self.db.profile.objectiveProgressSound) then
			PlaySoundFile(LSM:Fetch("sound", self.db.profile.objectiveProgressSoundFile))
		end
	end
	
	if(self.db.profile.echoQuestProgressToParty and GetNumPartyMembers() > 0 and GetNumRaidMembers() == 0) then
		SendChatMessage(desc .. ": " .. got .. "/" .. needed, "PARTY")
	end
end

function Inquest:QUEST_PROGRESS()
	if(self.db.profile.autoCompleteQuests) then
		CompleteQuest()
	end
end

function Inquest:QUEST_COMPLETE()
	if(self.db.profile.autoCompleteQuests and GetNumQuestChoices() == 0) then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function Inquest:CanWatchQuest(questIndex, zoneWatch)
	if(GetNumQuestLeaderBoards(questIndex) == 0 and not self.db.profile.trackWithNoObjectives) then
		if(not zoneWatch) then UIErrorsFrame:AddMessage(QUEST_WATCH_NO_OBJECTIVES, 1.0, 0.1, 0.1, 1.0) end
		return 0
	elseif((not WatchFrame:IsUserPlaced()) and ArenaEnemyFrames and ArenaEnemyFrames:IsShown()) then
		UIErrorsFrame:AddMessage(OBJECTIVES_WATCH_QUESTS_ARENA, 1.0, 0.1, 0.1, 1.0)
		return -1
	elseif(WatchFrame_GetRemainingSpace() < WatchFrame_GetHeightNeededForQuest(questIndex)) then
		UIErrorsFrame:AddMessage(OBJECTIVES_WATCH_TOO_MANY, 1.0, 0.1, 0.1, 1.0)
		return -1
	end
	
	return 1
end

function Inquest:QuestLogTitleButton_OnClick(titleButton, mouseButton)
	local questName = titleButton:GetText()
	local questIndex = titleButton:GetID() + FauxScrollFrame_GetOffset(QuestLogListScrollFrame)
	if(IsModifiedClick()) then
		if(titleButton.isHeader and IsModifiedClick("QUESTWATCHTOGGLE")) then
			local allWereTracked = true
			local zoneQuestIndex = questIndex + 1
			local currentTitle = _G["QuestLogTitle" .. zoneQuestIndex]
			while(currentTitle and not (currentTitle.isHeader) and currentTitle:IsShown()) do
				if(not IsQuestWatched(zoneQuestIndex)) then
					local canWatch = self:CanWatchQuest(zoneQuestIndex, true)
					if(canWatch == 1) then
						allWereTracked = false
						AddQuestWatch(zoneQuestIndex)
					elseif(canWatch == -1) then
						allWereTracked = false
						break
					end
				end
				
				zoneQuestIndex = zoneQuestIndex + 1
				currentTitle = _G["QuestLogTitle" .. zoneQuestIndex]
			end
			
			if(allWereTracked) then
				for qi=questIndex + 1, zoneQuestIndex - 1 do
					RemoveQuestWatch(qi)
				end
			end
			
			WatchFrame_Update()
			QuestLog_Update()
			return
		end
		
		if(IsModifiedClick("CHATLINK") and ChatFrameEditBox:IsVisible()) then
			local questLink = GetQuestLink(questIndex)
			if(questLink) then ChatEdit_InsertLink(questLink) end
		elseif(IsModifiedClick("QUESTWATCHTOGGLE")) then
			if(IsQuestWatched(questIndex)) then
				RemoveQuestWatch(questIndex)
				WatchFrame_Update()
			elseif(self:CanWatchQuest(questIndex) == 1) then
				AddQuestWatch(questIndex)
				WatchFrame_Update()
			end
		end
	end
	QuestLog_SetSelection(questIndex)
	QuestLog_Update()
end

function Inquest:QuestLog_Update()
	local numEntries, numQuests = GetNumQuestLogEntries()
	local offset = FauxScrollFrame_GetOffset(QuestLogListScrollFrame)
	
	self:UpdateLdbText()
	
	for i=1, QUESTS_DISPLAYED do
		local questIndex = i + offset
		local questLogTitle = _G["QuestLogTitle" .. i]
		local questTitleTag = _G["QuestLogTitle"..i.."Tag"]
		local questNormalText = _G["QuestLogTitle" .. i .. "NormalText"]
		local questCheck = _G["QuestLogTitle"..i.."Check"]
		
		--Set the new, tagged title for the quest
		if(questIndex <= numEntries) then
			local title = self:GetQuestLogTaggedTitle(questIndex)
			if(title) then
				questLogTitle:SetText(" " .. title)
				QuestLogDummyText:SetText(" " .. title)
			end
		end
		
		--Reposition the quest check based on the new length
		local tempWidth = 275 - 15 - questTitleTag:GetWidth()
		local textWidth = math.min(tempWidth, QuestLogDummyText:GetWidth())
		if(IsQuestWatched(questIndex)) then
			if(not questTitleTag:GetText() or questTitleTag:GetText() == "") then
				if(questNormalText:GetWidth() + 24 < 275) then
					questCheck:SetPoint("LEFT", questLogTitle, "LEFT", questNormalText:GetWidth() + 24, 0)
				else
					questCheck:SetPoint("LEFT", questLogTitle, "LEFT", questNormalText:GetWidth() + 10, 0)
				end
			else
				if(questNormalText:GetWidth() + 24 < 275) then
					questCheck:SetPoint("LEFT", questLogTitle, "LEFT", textWidth + 24, 0)
				else
					questCheck:SetPoint("LEFT", questLogTitle, "LEFT", textWidth + 10, 0)
				end
			end
		end
	end
end

function Inquest:SetQuestLogPosition()
	if(QuestLogFrame.moving) then return end
	
	QuestLogFrame:ClearAllPoints()
	if(self.db.profile.logxOffset and self.db.profile.logyOffset) then
		QuestLogFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.logxOffset, self.db.profile.logyOffset)
	else
		QuestLogFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, -104)
	end
end

function Inquest:SetQuestLogLocked(questLogLocked)
	questLogLocked = questLogLocked or self.db.profile.questLogLocked
	if(questLogLocked) then
		QuestLogFrame:RegisterForDrag(nil)
	else
		QuestLogFrame:RegisterForDrag("LeftButton")
	end
end

function Inquest:SaveQuestLogPosition()
	self.db.profile.logxOffset = QuestLogFrame:GetLeft()
	self.db.profile.logyOffset = QuestLogFrame:GetTop()
end

function Inquest:QuestLog_OnDragStart()
	QuestLogFrame.moving = true
	QuestLogFrame:StartMoving()
end

function Inquest:QuestLog_OnDragStop()
	QuestLogFrame.moving = false
	QuestLogFrame:StopMovingOrSizing()
	self:SaveQuestLogPosition()
end
