--[[
	Note: for the purposes of this table, this is the lookup. This may be accessed with the routine:
	faction = ReagentData_GetFaction(recipe)
		if the recipe does not require faction, returns nil
		otherwise, returns a table containing records of the form:
			table[<#>] = {
				faction = <faction name>,
				standing = <standing required>,
			}
	This is necessary because certain recipes are provided by two different factions (in
	particular, Thrallmar/Honor Hold).

 -3 = Hated
 -2 = Hostile
 -1 = Unfriendly
 0 = Neutral
 1 = Friendly
 2 = Honored
 3 = Revered
 4 = Exalted
]]

-- Faction Classes
local CLASS_SHATTRATH = "Shattrath City";
local CLASS_AZEROTH = "Azeroth";
local CLASS_AZEROTHBC = "Azeroth/BC";
local CLASS_OUTLAND = "Outland";

function ReagentData_Faction(num)
	if (num == 4) then
		return "Exalted";
	elseif (num == 3) then
		return "Revered";
	elseif (num == 2) then
		return "Honored";
	elseif (num == 1) then
		return "Friendly";
	elseif (num == 0) then
		return "Neutral";
	elseif (num == -1) then
		return "Unfriendly";
	elseif (num == -2) then
		return "Hostile";
	elseif (num == -3) then
		return "Hated";
	else
		return "Unknown";
	end
end

function ReagentData_GetFaction(recipe)
	local lfaction, count, tfac;

	lfaction = {};
	count = 0;
	for k, v in pairs(ReagentData["faction"]) do
		if (type(v) == "table") then
			for krecipe, kstanding in pairs(v) do
				if (krecipe == recipe) then
					tfac = {};
					
					tfac.faction = k;
					tfac.standing = ReagentData_Faction(kstanding);
					table.insert(lfaction, tfac);
					count = count + 1;
				end
			end
		end
	end
	if (count > 0) then
		return lfaction;
	else
		return nil;
	end
end

ReagentData["faction"] = {
	["The Aldor"] = {
		class = CLASS_SHATTRATH,
		-- Blacksmithing
		["Flamebane Bracers"] = 1,
		["Flamebane Breastplate"] = 3,
		["Flamebane Gloves"] = 2,
		["Flamebane Helm"] = 4,
		-- Jewelcrafting
		["Gleaming Golden Draenite"] = 1,
		["Pendant of Shadow's End"] = 3,
		["Royal Shadow Draenite"] = 2,
		-- Leatherworking
		["Blastguard Belt"] = 2,
		["Blastguard Boots"] = 3,
		["Blastguard Pants"] = 4,
		["Flamescale Belt"] = 2,
		["Flamescale Boots"] = 3,
		["Flamescale Leggings"] = 4,
		["Vindicator's Armor Kit"] = 3,
		-- Tailoring
		["Flameheart Bracers"] = 1,
		["Flameheart Gloves"] = 2,
		["Flameheart Vest"] = 4,
		["Golden Spellthread"] = 4,
		["Silver Spellthread"] = 2,
	},
	["Argent Dawn"] = {
		class = CLASS_AZEROTH,
		-- Alchemy
		["Transmute: Air to Fire"] = 2,
		-- Blacksmithing
		["Girdle of the Dawn"] = 2,
		["Gloves of the Dawn"] = 3,
		["Icebane Bracers"] = 3,
		["Icebane Breastplate"] = 4,
		["Icebane Gauntlets"] = 3,
		-- Enchanting
		["Enchant Bracer - Healing"] = 3,
		["Enchant Bracer - Mana Regeneration"] = 2,
		-- First Aid
		["Powerful Anti-Venom"] = 2,
		-- Leatherworking
		["Dawn Treaders"] = 2,
		["Golden Mantle of the Dawn"] = 3,
		["Icy Scale Bracers"] = 3,
		["Icy Scale Breastplate"] = 4,
		["Icy Scale Gauntlets"] = 3,
		["Polar Bracers"] = 3,
		["Polar Gloves"] = 3,
		["Polar Tunic"] = 4,
		-- Tailoring
		["Argent Boots"] = 2,
		["Argent Shoulders"] = 3,
		["Glacial Cloak"] = 4,
		["Glacial Gloves"] = 3,
		["Glacial Wrists"] = 3,
		["Glacial Vest"] = 4,
	},
	["Ashtongue Deathsworn"] = {
	    class = CLASS_OUTLAND,
	    -- Blacksmithing
	    ["Shadesteel Bracers"] = 1,
	    ["Shadesteel Girdle"] = 1,
	    ["Shadesteel Greaves"] = 2,
	    ["Shadesteel Sabots"] = 2,
	    -- Leatherworking
	    ["Boots of Shackled Souls"] = 1,
	    ["Bracers of Shackled Souls"] = 1,
	    ["Redeemed Soul Cinch"] = 1,
	    ["Redeemed Soul Legguards"] = 1,
	    ["Greaves of Shackled Souls"] = 2,
	    ["Redeemed Soul Mocassins"] = 2,
	    ["Redeemed Soul Wristguards"] = 2,
	    ["Waistguard of Shackled Souls"] = 2,
	    -- Tailoring
	    ["Soulguard Bracers"] = 1,
	    ["Soulguard Girdle"] = 1,
	    ["Night's End"] = 2,
	    ["Soulguard Leggings"] = 2,
	    ["Soulguard Slippers"] = 2,
	},
	["Cenarion Circle"] = {
		class = CLASS_AZEROTH,
		-- Blacksmithing
		["Heavy Obsidian Belt"] = 1,
		["Ironvine Belt"] = 1,
		["Ironvine Breastplate"] = 3,
		["Ironvine Gloves"] = 2,
		["Jagged Obsidian Shield"] = 3,
		["Light Obsidian Belt"] = 2,
		["Obsidian Mail Tunic"] = 4,
		-- Enchanting
		["Enchant Cloak - Greater Fire Resistance"] = 1,
		["Enchant Cloak - Greater Nature Resistance"] = 2,
		-- Leatherworking
		["Bramblewood Belt"] = 1,
		["Bramblewood Boots"] = 2,
		["Bramblewood Helm"] = 3,
		["Dreamscale Breastplate"] = 4,
		["Sandstalker Bracers"] = 1,
		["Sandstalker Breastplate"] = 3,
		["Sandstalker Gauntlets"] = 2,
		["Spitfire Bracers"] = 1,
		["Spitfire Breastplate"] = 3,
		["Spitfire Gauntlets"] = 2,
		-- Tailoring
		["Cenarion Herb Bag"] = 1,
		["Gaea's Embrace"] = 3,
		["Satchel of Cenarius"] = 3,
		["Sylvan Crown"] = 2,
		["Sylvan Shoulders"] = 1,
		["Sylvan Vest"] = 3,
	},
	["Cenarion Expedition"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Flask of Distilled Wisdom"] = 4,
		["Major Nature Protection Potion"] = 4,
		["Transmute: Earthstorm Diamond"] = 2,
		["Earthen Elixir"] = 2,
		["Transmute: Primal Water to Air"] = 3,
		-- Blacksmithing
		["Adamantite Sharpening Stone"] = 2,
		["Adamantite Weightstone"] = 2,
		["Greater Rune of Warding"] = 2,
		["Wildguard Breastplate"] = 4,
		["Wildguard Helm"] = 3,
		["Wildguard Leggings"] = 3,
		-- Enchanting
		["Enchant Gloves - Spell Strike"] = 3,
		-- Engineering
		["Green Smoke Flare"] = 1,
		-- Jewelcrafting
		["Figurine - Nightseye Panther"] = 3,
		["The Natural Ward"] = 4,
		-- Leatherworking
		["Clefthide Leg Armor"] = 2,
		["Heavy Clefthoof Boots"] = 1,
		["Heavy Clefthoof Leggings"] = 2,
		["Heavy Clefthoof Vest"] = 2,
		["Nethercleft Leg Armor"] = 4,
		-- Tailoring
		--["Heavy Netherweave Net"] = 1,
	},
	["The Consortium"] = {
		class = CLASS_OUTLAND,
		-- Enchanting
		["Enchant Cloak - Spell Penetration"] = 1,
		["Enchant Weapon - Major Striking"] = 2,
		["Enchant Ring - Striking"] = 3,
		-- Engineering
		["Elemental Seaforium Charge"] = 3,
		-- Jewelcrafting
		["Delicate Blood Garnet"] = 2,
		["Luminous Flame Spessarite"] = 1,
		["Lustrous Azure Moonstone"] = 2,
		["Pendant of the Null Rune"] = 3,
		["Powerful Earthstorm Diamond"] = 2,
		["Shifting Shadow Draenite"] = 1,
		["Swift Skyfire Diamond"] = 2,
		["Thick Golden Draenite"] = 2,
		["Bracing Earthstorm Diamond"] = 3,
		["Relentless Earthstorm Diamond"] = 3,
		-- Leatherworking
		["Fel Leather Boots"] = 2,
		["Fel Leather Gloves"] = 1,
		["Fel Leather Leggings"] = 3,
		-- Tailoring
		["Bag of Jewels"] = 2,
	},
	["Honor Hold"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Elixir of Major Agility"] = 2,
		["Transmute: Skyfire Diamond"] = 2,
		-- Blacksmithing
		["Felsteel Shield Spike"] = 4,
		-- Enchanting
		["Enchant Bracer - Superior Healing"] = 1,
		["Enchant Chest - Exceptional Stats"] = 3,
		-- Jewelcrafting
		["Enduring Deep Peridot"] = 1,
		["Figurine - Dawnstone Crab"] = 3,
		-- Leatherworking
		["Cobrahide Leg Armor"] = 2,
		["Felstalker Belt"] = 1,
		["Felstalker Bracers"] = 2,
		["Felstalker Breastplate"] = 2,
		["Nethercobra Leg Armor"] = 4,
	},
	["Keepers of Time"] = {
		class = CLASS_AZEROTHBC,
		-- Alchemy
		["Flask of Supreme Power"] = 4,
		-- Enchanting
		["Enchant Gloves - Major Spellpower"] = 2,
		["Enchant Ring - Spellpower"] = 2,
		-- Jewelcrafting
		["Enigmatic Skyfire Diamond"] = 2,
		["Figurine - Living Ruby Serpent"] = 3,
		["Pendant of Frozen Flame"] = 3,
		-- Leatherworking
		["Drums of Panic"] = 4,
	},
	["Kurenai"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Transmute: Primal Fire to Earth"] = 3,
		-- Leatherworking
		["Netherfury Belt"] = 1,
		["Netherfury Boots"] = 3,
		["Netherfury Leggings"] = 2,
		["Reinforced Mining Bag"] = 2,
	},
	["Lower City"] = {
		class = CLASS_SHATTRATH,
		-- Alchemy
		["Elixir of Major Shadow Power"] = 3,
		["Flask of Chromatic Resistance"] = 4,
		-- Jewelcrafting
		["Figurine - Felsteel Boar"] = 3,
		["Pendant of Thawing"] = 3,
		["Potent Flame Spessarite"] = 1,
		-- Tailoring
		["Cloak of Arcane Evasion"] = 2,
	},
	["The Mag'har"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Transmute: Primal Fire to Earth"] = 3,
		-- Leatherworking
		["Netherfury Belt"] = 1,
		["Netherfury Boots"] = 3,
		["Netherfury Leggings"] = 2,
		["Reinforced Mining Bag"] = 2,
	},
	["The Scale of the Sands"] = {
		class = CLASS_AZEROTHBC,
		-- Enchanting
		["Enchant Ring - Stats"] = 3,
		-- Jewelcrafting
		["Bold Crimson Spinel"] = 1,
		["Bright Crimson Spinel"] = 1,
		["Brilliant Lionseye"] = 1,
		["Delicate Crimson Spinel"] = 1,
		["Gleaming Lionseye"] = 1,
		["Lustrous Empyrean Sapphire"] = 1,
		["Runed Crimson Spinel"] = 1,
		["Smooth Lionseye"] = 1,
		["Solid Empyrean Sapphire"] = 1,
		["Sparkling Empyrean Sapphire"] = 1,
		["Subtle Crimson Spinel"] = 1,
		["Teardrop Crimson Spinel"] = 1,
		["Thick Lionseye"] = 1,
		["Balanced Shadowsong Amethyst"] = 2,
		["Dazzling Seaspray Emerald"] = 2,
		["Glinting Pyrestone"] = 2,
		["Glowing Shadowsong Amethyst"] = 2,
		["Jagged Seaspray Emerald"] = 2,
		["Luminous Pyrestone"] = 2,
		["Potent Pyrestone"] = 2,
		["Radiant Seaspray Emerald"] = 2,
		["Enduring Seaspray Emerald"] = 3,
		["Royal Shadowsong Amethyst"] = 3,
		["Wicked Pyrestone"] = 3,
	},
	["The Scryers"] = {
		class = CLASS_SHATTRATH,
		-- Alchemy
		["Elixir of Major Firepower"] = 3,
		-- Blacksmithing
		["Enchanted Adamantite Belt"] = 1,
		["Enchanted Adamantite Boots"] = 2,
		["Enchanted Adamantite Breastplate"] = 3,
		["Enchanted Adamantite Leggings"] = 4,
		-- Jewelcrafting
		["Dazzling Deep Peridot"] = 2,
		["Pendant of Withering"] = 3,
		["Runed Blood Garnet"] = 1,
		-- Leatherworking
		["Enchanted Clefthoof Boots"] = 2,
		["Enchanted Clefthoof Gloves"] = 3,
		["Enchanted Clefthoof Leggings"] = 4,
		["Enchanted Felscale Boots"] = 3,
		["Enchanted Felscale Gloves"] = 2,
		["Enchanted Felscale Leggings"] = 4,
		["Magister's Armor Kit"] = 3,
		-- Tailoring
		["Mystic Spellthread"] = 2,
		["Runic Spellthread"] = 4,
	},
	["The Sha'tar"] = {
		class = CLASS_SHATTRATH,
		-- Alchemy
		["Alchemist's Stone"] = 3,
		["Flask of the Titans"] = 4,
		["Transmute: Primal Air to Fire"] = 3,
		-- Enchanting
		["Enchant Gloves - Major Healing"] = 2,
		["Enchant Ring - Healing Power"] = 3,
		["Enchant Weapon - Major Healing"] = 3,
		-- Jewelcrafting
		["Insightful Earthstorm Diamond"] = 1,
		["Ring of Arcane Shielding"] = 2,
		["Figurine - Talasite Owl"] = 3,
		-- Leatherworking
		["Drums of Battle"] = 4,
	},
	["Sporregar"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Shrouding Potion"] = 4,
		["Transmute: Primal Earth to Water"] = 3,
		-- Cooking
		["Clam Bar"] = 1,
		["Sporeling Snack"] = 1,
	},
	["Thrallmar"] = {
		class = CLASS_OUTLAND,
		-- Alchemy
		["Elixir of Major Agility"] = 2,
		["Transmute: Skyfire Diamond"] = 2,
		-- Blacksmithing
		["Felsteel Shield Spike"] = 4,
		-- Enchanting
		["Enchant Bracer - Superior Healing"] = 1,
		["Enchant Chest - Exceptional Stats"] = 3,
		-- Jewelcrafting
		["Enduring Deep Peridot"] = 1,
		["Figurine - Dawnstone Crab"] = 3,
		-- Leatherworking
		["Cobrahide Leg Armor"] = 2,
		["Felstalker Belt"] = 1,
		["Felstalker Bracers"] = 2,
		["Felstalker Breastplate"] = 2,
		["Nethercobra Leg Armor"] = 4,
	},
	["Thorium Brotherhood"] = {
		class = CLASS_AZEROTH,
		-- Alchemy
		["Transmute: Elemental Fire"] = 1,
		-- Blacksmithing
		["Black Amnesty"] = 3,
		["Blackfury"] = 3,
		["Blackguard"] = 4,
		["Dark Iron Boots"] = 4,
		["Dark Iron Bracers"] = 1,
		["Dark Iron Destroyer"] = 2,
		["Dark Iron Gauntlets"] = 3,
		["Dark Iron Helm"] = 2,
		["Dark Iron Leggings"] = 3,
		["Dark Iron Reaver"] = 2,
		["Ebon Hand"] = 4,
		["Fiery Chain Girdle"] = 2,
		["Fiery Chain Shoulders"] = 3,
		["Nightfall"] = 4,
		-- Enchanting
		["Enchant Weapon - Mighty Intellect"] = 3,
		["Enchant Weapon - Mighty Spirit"] = 2,
		["Enchant Weapon - Strength"] = 1,
		-- Leatherworking
		["Black Dragonscale Boots"] = 2,
		["Chromatic Gauntlets"] = 3,
		["Corehound Belt"] = 3,
		["Corehound Boots"] = 1,
		["Lava Belt"] = 2,
		["Molten Belt"] = 3,
		["Molten Helm"] = 1,
		-- Tailoring
		["Flarecore Gloves"] = 1,
		["Flarecore Leggings"] = 3,
		["Flarecore Mantle"] = 2,
		["Flarecore Robe"] = 2,
	},
	["Timbermaw Hold"] = {
		class = CLASS_AZEROTH,
		-- Alchemy
		["Transmute: Earth to Water"] = 1,
		-- Blacksmithing
		["Heavy Timbermaw Belt"] = 2,
		["Heavy Timbermaw Boots"] = 3,
		-- Enchanting
		["Enchant 2H Weapon - Agility"] = 1,
		["Enchant Weapon - Agility"] = 2,
		-- Leatherworking
		["Might of the Timbermaw"] = 2,
		["Timbermaw Brawlers"] = 3,
		["Warbear Harness"] = 2,
		["Warbear Woolies"] = 1,
		-- Tailoring
		["Mantle of the Timbermaw"] = 3,
		["Wisdom of the Timbermaw"] = 2,
	},
	["The Violet Eye"] = {
		class = CLASS_AZEROTHBC,
		-- Blacksmithing
		["Iceguard Breastplate"] = 2,
		["Iceguard Helm"] = 2,
		["Iceguard Leggings"] = 3,
		-- Jewelcrafting
		["The Frozen Eye"] = 2,
	},
	["Zandalar Tribe"] = {
		class = CLASS_AZEROTH,
		-- Alchemy
		["Greater Dreamless Sleep Potion"] = 1,
		["Living Action Potion"] = 4,
		["Mageblood Potion"] = 3,
		["Major Troll's Blood Potion"] = 2,
		-- Blacksmithing
		["Bloodsoul Breastplate"] = 3,
		["Bloodsoul Gauntlets"] = 1,
		["Bloodsoul Shoulders"] = 2,
		["Darksoul Breastplate"] = 3,
		["Darksoul Leggings"] = 2,
		["Darksoul Shoulders"] = 1,
		-- Enchanting
		["Brilliant Mana Oil"] = 1,
		["Brilliant Wizard Oil"] = 2,
		-- Engineering
		["Bloodvine Goggles"] = 2,
		["Bloodvine Lens"] = 1,
		-- Leatherworking
		["Blood Tiger Breastplate"] = 3,
		["Blood Tiger Shoulders"] = 2,
		["Primal Batskin Bracers"] = 1,
		["Primal Batskin Gloves"] = 2,
		["Primal Batskin Jerkin"] = 3,
		-- Tailoring
		["Bloodvine Boots"] = 1,
		["Bloodvine Leggings"] = 2,
		["Bloodvine Vest"] = 3,
	},
}

