﻿Tabard_O_Matic = LibStub("AceAddon-3.0"):NewAddon("Tabard_O_Matic", "AceEvent-3.0", "AceHook-3.0")
local TEXTURE_LINK_FORMAT = "|T%s:%d:%d:0:%d|t"

local containerMax = 15
local bagMax = 30
local bankMax = 40
local _G = getfenv(0)

function Tabard_O_Matic:OnInitialize()
	if LoadAddOn("Baggins") then
		SetUpBaggins()
	end
	
	if LoadAddOn("Combuctor") then
		SetUpCombuctor()
	end
	
	if LoadAddOn("Baudbag") then
		SetUpBaudbag()
	end	
	
	if LoadAddOn("ArkInventory") then
		SetUpArkInventory()
	end	
	
	if LoadAddOn("TBag") then
		SetUpTBag()
	end	
	
	if LoadAddOn("AtlasLoot") then
		SetUpAtlasLoot()
	end

	if LoadAddOn("Bagnon") then
		SetUpBagnon()
	end	

	if LoadAddOn("FBagOfHolding") then
		SetUpFBagOfHolding_Bank()
		SetUpFBagOfHolding_Containier()
		SetUpFBagOfHolding_AltContainier()
	end		
end

function Tabard_O_Matic:OnEnable()
	local TOC_Version = select(4, GetBuildInfo())
	
	-- Bag/Bank Hooks
	self:SecureHook("ContainerFrame_OnShow", "UpdateAllBags")
	self:SecureHook("ContainerFrame_Update")
	self:SecureHook("BankFrameItemButton_Update", "UpdateAllBags")
	self:SecureHook("BankFrame_OnShow", "UpdateAllBags")

	-- Merchant Hooks
	self:SecureHook("MerchantFrame_Update")
	self:SecureHook("MerchantFrame_OnShow", "MerchantFrame_Update")
	self:SecureHook("MerchantFrame_UpdateBuybackInfo", "MerchantFrame_Update")
	self:SecureHook("MerchantPrevPageButton_OnClick", "MerchantFrame_Update")
	self:SecureHook("MerchantNextPageButton_OnClick", "MerchantFrame_Update")
	
	-- Paperdoll Hooks
	self:SecureHook("PaperDollItemSlotButton_Update", "UpdateItems")	
	self:SecureHook("CharacterFrame_OnShow", "UpdateItems")
	self:SecureHook("CharacterFrameTab_OnClick", "UpdateItems")
	
	if TOC_Version >= 30100 then
		self:SecureHook("PaperDollFrameItemFlyout_DisplayButton", "FlyoutUpdateItems")
	end
end

function Tabard_O_Matic:SetTheButtons(Button, TheTabard)
	SetItemButtonTexture(Button, [[Interface\Addons\Tabard-o-matic\Tabards\]] .. TheTabard)
end

function Tabard_O_Matic:FlyoutUpdateItems(button, paperDollItemSlot)
	local location = button.location
	
	if (location < PDFITEMFLYOUT_FIRST_SPECIAL_LOCATION) then
		local ItemID, name, textureName, count, durability, maxDurability, InventoryType, locked, start, duration, enable, setTooltip, gem1, gem2, gem3 = EquipmentManager_GetItemInfoByLocation(location)
		
		if InventoryType == "INVTYPE_TABARD" then
			local TabardValue = TabardTextures[ItemID]
				
			if TabardValue then
				Tabard_O_Matic:SetTheButtons(button, TabardValue.ItemName)
			end
		end
	end
end

function Tabard_O_Matic:MerchantFrame_Update()
	if (MerchantFrame.selectedTab == 1) and MerchantFrame:IsVisible() then
		local numMerchantItems = GetMerchantNumItems()
		for i = 1, MERCHANT_ITEMS_PER_PAGE, 1 do
			local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i)
			local itemButton = getglobal("MerchantItem"..i.."ItemButton")
			if ( index <= numMerchantItems ) then
				local MerchantLink = GetMerchantItemLink(index)
				if MerchantLink then
					local ItemID = tonumber(MerchantLink:match("item:(%d+)"))
					local TabardValue = TabardTextures[ItemID]
					
					if TabardValue then
						Tabard_O_Matic:SetTheButtons(itemButton, TabardValue.ItemName)
					end
				end
			end
		end
		
		local BuyBackLink = GetBuybackItemLink(GetNumBuybackItems())		
		if BuyBackLink then
			local ItemID = tonumber(BuyBackLink:match("item:(%d+)"))
			for TabardID, TabardValue in pairs (TabardTextures) do
				if ItemID == TabardID then
					Tabard_O_Matic:SetTheButtons(MerchantBuyBackItemItemButton, TabardValue.ItemName)
				end
			end	
		end
	else
		local numBuybackItems = GetNumBuybackItems()
		for i = 1, BUYBACK_ITEMS_PER_PAGE do
			local itemButton = getglobal("MerchantItem"..i.."ItemButton")
			local buybackButton = getglobal("MerchantItem"..i)
			if ( i <= numBuybackItems ) then
				local BuyBackLink = GetBuybackItemLink(i)
				if BuyBackLink then
					local ItemID = tonumber(BuyBackLink:match("item:(%d+)"))
					local TabardValue = TabardTextures[ItemID]
					
					if TabardValue then
						Tabard_O_Matic:SetTheButtons(itemButton, TabardValue.ItemName)
					end
				end
			end
		end
	end
end

function Tabard_O_Matic:UpdateItems()
	if CharacterFrame:IsVisible() then
		local TabardLink = GetInventoryItemLink("player", GetInventorySlotInfo("TabardSlot"))
		if TabardLink then
			local ItemID = tonumber(TabardLink:match("item:(%d+)"))
			local TabardValue = TabardTextures[ItemID]
			
			if TabardValue then
				Tabard_O_Matic:SetTheButtons(CharacterTabardSlot, TabardValue.ItemName)
			end		
		end
	end
end

function Tabard_O_Matic:UpdateBankAndBags()
	for containerIndex = 1, containerMax do
		local bagFrame = _G["ContainerFrame"..containerIndex]
		if (bagFrame) then
			local bagIndex = bagFrame:GetID()
			if (bagFrame:IsVisible()) then
				for itemIndex = 1, bagMax do
					local slotFrame = _G["ContainerFrame" .. containerIndex .. "Item" .. itemIndex]
					if (slotFrame) then
						local slotIndex = abs(itemIndex - (GetContainerNumSlots(bagIndex) + 1))
						if (slotFrame:IsVisible()) then
							local ItemLink = GetContainerItemLink(bagIndex, slotIndex)
							if (ItemLink) then
								local ItemID = tonumber(ItemLink:match("item:(%d+)"))
								local TabardValue = TabardTextures[ItemID]

								if TabardValue then
									Tabard_O_Matic:SetTheButtons(slotFrame, TabardValue.ItemName)
								end
							end
						end
					end
				end
			end
		end
	end

	for bankIndex = 1, bankMax do
		local slotFrame = _G["BankFrameItem" .. bankIndex]
		if ((slotFrame) and (slotFrame:IsVisible())) then
			local ItemLink = GetContainerItemLink(-1, bankIndex)
			if (ItemLink) then
				local ItemID = tonumber(ItemLink:match("item:(%d+)"))
				local TabardValue = TabardTextures[ItemID]

				if TabardValue then
					Tabard_O_Matic:SetTheButtons(slotFrame, TabardValue.ItemName)
				end
			end
		end
	end	
end

function Tabard_O_Matic:UpdateAllBags()
	self:UpdateBankAndBags()
end

function Tabard_O_Matic:ContainerFrame_Update(frame)
	local id = frame:GetID();
	local name = frame:GetName();
	local itemButton;
	local texture, itemCount, locked, quality, readable;
	local tooltipOwner = GameTooltip:GetOwner();
	for i=1, frame.size, 1 do
		itemButton = getglobal(name.."Item"..i);
		
		texture, itemCount, locked, quality, readable = GetContainerItemInfo(id, itemButton:GetID());
		
		SetItemButtonTexture(itemButton, texture);
		SetItemButtonCount(itemButton, itemCount);
		
		local ItemLink = GetContainerItemLink(id, itemButton:GetID())
		if (ItemLink) then
			local ItemID = tonumber(ItemLink:match("item:(%d+)"))
			local TabardValue = TabardTextures[ItemID]

			if TabardValue then
				Tabard_O_Matic:SetTheButtons(itemButton, TabardValue.ItemName)
			end
		end	
		
		SetItemButtonDesaturated(itemButton, locked, 0.5, 0.5, 0.5);

		if ( texture ) then
			ContainerFrame_UpdateCooldown(id, itemButton);
			itemButton.hasItem = 1;
		else
			getglobal(name.."Item"..i.."Cooldown"):Hide();
			itemButton.hasItem = nil;
		end
		itemButton.readable = readable;
		
		if ( itemButton == tooltipOwner ) then
			itemButton.UpdateTooltip(itemButton);
		end		
	end
end