-- Copyright  2008, 2009 Xianghar  <xian@zron.de>
-- All Rights Reserved.
-- This code is not to be modified or distributed without written permission by the author.
-- Current permissions only include curse.com, wowui.worldofwar.net, wowinterface.com and their respective addon updaters

local TRACKER_START = 5
local TRACKER_END = 8
local Timers = TotemTimers.timers

local Icons = TotemTimers_Icons

local TotemFrameScript = TotemFrame:GetScript("OnShow")

local TotemColors = {
    [AIR_TOTEM_SLOT] = {0.7,0.7,1.0},
    [WATER_TOTEM_SLOT] = {0.4,0.4,1.0},
    [FIRE_TOTEM_SLOT] = {1.0,0.1,0.1},
    [EARTH_TOTEM_SLOT] = {0.7,0.5,0.3},
}

local warningColors = {
	["warning"] = {
		["color"] = {r=1.0, g=0.5, b=0.0},
		["defColor"] = {r=1.0, g=1.0, b=0.0},
	},
	["expiration"] = {
		["color"] = {r=1.0, g=0.0, b=0.0},
		["defColor"] = {r=0.0, g=1.0, b=0.0},
	},
}


local defmsgfunc = function(self, warning, type, icon)
	UIErrorsFrame:AddMessage(warning,warningColors[type].defColor.r, warningColors[type].defColor.g,
			warningColors[type].defColor.b, 1.0, UIERRORS_HOLD_TIME);
end


local function SetCastButtonsState(alwaysshow)
    local Timers = TotemTimers.timers
    for i = 1,4 do
        for j = 1,7 do
            Timers[i].CastButtons[j]:SetAttribute("alwaysshow",alwaysshow)
        end
    end
end


local SettingsFunctions

function TotemTimers_ProcessSetting(setting)
    if SettingsFunctions[setting] then
        SettingsFunctions[setting](TotemTimers_Settings[setting], TotemTimers.timers)
    end
end


SettingsFunctions = {
    ShowTimerBars = 
        function(value, Timers) 
            for _,timer in pairs(Timers) do
                timer.visibleTimerBars = value
                for n,t in pairs(timer.timers) do
                    if t>0 and value then
                        timer:ShowTimerBar(n)
                    else
                        timer.timerbars[n].background:Hide()
                        timer.timerbars[n]:SetValue(0)
                    end					
                end
            end   
        end,

    FlashRed = 
        function(value, Timers)
        	for _,timer in pairs(Timers) do
                timer.FlashRed = value
            end
        end,
        
    TimerSize = 
        function(value, Timers)
            local v = value
            if Timers[5].active then v = v / (TotemTimers_Settings.TrackerSize/36) end
    		for e=1,4 do
    			Timers[e]:SetScale(v/36)
    		end
        end,
        
    TrackerSize = 
        function(value, Timers)
    		for e=TRACKER_START, TRACKER_END do
    			Timers[e]:SetScale(value/36)
    		end
            TotemTimers_ProcessSetting("TimerSize")
        end,

    TimerTimeHeight =
        function(value, Timers)
    		for e=1,4 do
    			Timers[e]:SetTimeHeight(value)
    		end        
        end,
        
    TrackerTimeHeight =
        function(value, Timers)
    		for e=TRACKER_START, TRACKER_END do
    			Timers[e]:SetTimeHeight(value)
    		end
        end,
        
    TimerSpacing = 
        function(value, Timers)
    		for e=1,4 do
    			Timers[e]:SetSpacing(value)
    		end
        end,
        
    TrackerSpacing =
        function(value, Timers)
    		for e=TRACKER_START, TRACKER_END do
    			Timers[e]:SetSpacing(value)
    		end
        end,
        
    TimerTimeSpacing = 
        function(value, Timers)
    		for e=1,4 do
    			Timers[e]:SetTimeSpacing(value)
    		end
        end,
        
    TrackerTimeSpacing =
        function(value, Timers)
    		for e=TRACKER_START, TRACKER_END do
    			Timers[e]:SetTimeSpacing(value)
    		end
        end,
        
    TimerTimePos = 
        function(value, Timers)
    		for e=1,4 do
    			Timers[e]:SetTimerBarPos(value)
    		end  
        end,
        
    TrackerTimePos =
        function(value, Timers)
    		for e=TRACKER_START, TRACKER_END do
    			Timers[e]:SetTimerBarPos(value)
    		end
        end,
    
    AnkhTracker = 
        function(value, Timers)
    		if value and TotemTimers_Spells[Icons.Reincarnation] then
    			Timers[5]:activate()
    		else
    			Timers[5]:deactivate()
    		end
    		TotemTimers_OrderTrackers()
        end,
        
    ShieldTracker =
        function(value, Timers)
    		if (value or TotemTimers_Settings.ActivateHiddenTimers) and TotemTimers_Spells[Icons.LightningShield] then
    			Timers[6]:activate()
    		else
    			Timers[6]:deactivate()
    		end
    		TotemTimers_OrderTrackers()		
        end,
        
    EarthShieldTracker = 
        function(value, Timers)
    		if (value or TotemTimers_Settings.ActivateHiddenTimers) and TotemTimers_Spells[Icons.EarthShield] then
    			Timers[7]:activate()
    		else
    			Timers[7]:deactivate()
    		end
    		TotemTimers_OrderTrackers()
        end,
        
    WeaponTracker =
        function(value, Timers)
    		if (value or TotemTimers_Settings.ActivateHiddenTimers) and TotemTimers_Spells[Icons.RockbiterWeapon] then
    			Timers[8]:activate()
    		else
    			Timers[8]:deactivate()
    		end
    		TotemTimers_OrderTrackers()
        end,
        
    Msg =
        function(value)
    		if value == "tt" then
    			TotemTimers.Msg = defmsgfunc
    		elseif value == "sct" then
    			TotemTimers.Msg =
    				function(self, warning, type, icon)
    					if not SCT then defmsgfunc(self, warning, type, icon) else
    						SCT:DisplayText(warning, warningColors[type].color, nil, nil, SCT.FRAME1, nil, nil, icon)
    					end
    				end
    		elseif value == "sct2" then
    			TotemTimers.Msg = 
    				function(self, warning, type, icon)
    					if not SCT then defmsgfunc(self, warning, type, icon) else
    						SCT:DisplayText(warning, warningColors[type].color, true, nil, SCT.FRAME1, nil,nil, icon)
    					end
    				end
    		elseif value == "msbt" then
    			TotemTimers.Msg = 
    				function(self, warning, type, icon)
    					if not MikSBT then defmsgfunc(self, warning, type, icon) else
    						MikSBT.DisplayMessage(warning, nil, nil, warningColors[type].color.r*255,warningColors[type].color.g*255,warningColors[type].color.b*255)
    					end
    				end
    		elseif value == "parrot" then
    			TotemTimers.Msg = 
    				function(self, warning, type, icon)
    					if not Parrot then defmsgfunc(self, warning, type, icon) else
    						Parrot:ShowMessage(warning, "Notification", false, warningColors[type].color.r,
    							warningColors[type].color.g, warningColors[type].color.b)
    					end
    				end
    		elseif value == "parrotcrit" then
    			TotemTimers.Msg = 
    				function(self, warning, type, icon)
    					if not Parrot then defmsgfunc(self, warning, type, icon) else
    						Parrot:ShowMessage(warning, "Notification", true, warningColors[type].color.r,
    							warningColors[type].color.g, warningColors[type].color.b)
    					end
    				end
            elseif value == "chat" then
    			TotemTimers.Msg =
    				function(self, warning, type, icon)
    					DEFAULT_CHAT_FRAME:AddMessage(warning, warningColors[type].defColor.r, warningColors[type].defColor.g, warningColors[type].defColor.b)
    				end
    		end
        end,
        
    HideBlizzTimers = 
        function(value)
            if value then
                TotemFrame:UnregisterEvent("PLAYER_TOTEM_UPDATE")
                TotemFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
                TotemFrame:SetScript("OnShow", function() TotemFrame:Hide() end)
                TotemFrame:Hide()
            else
                TotemFrame:RegisterEvent("PLAYER_TOTEM_UPDATE")
                TotemFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
                TotemFrame:Show()
                TotemFrame:SetScript("OnShow", TotemFrameScript)
            end
        end,
            
    ShieldLeftButton =
        function(value, Timers)
    		Timers[6].button:SetAttribute("*spell1",TotemTimers_Spells[value])
        end,
        
    ShieldRightButton =
        function(value, Timers)
    		Timers[6].button:SetAttribute("*spell2", TotemTimers_Spells[value])
        end,
            
    ShieldMiddleButton =
        function(value, Timers)
    		Timers[6].button:SetAttribute("*spell3", TotemTimers_Spells[value])
        end,

    EarthShieldLeftButton =
        function(value, Timers)
			if value ~= "recast" then
				Timers[7].button:SetAttribute("*unit1", value)
			else
				TotemTimers_EnableEarthShieldRecast()
			end
        end,
        
    EarthShieldRightButton =
        function(value, Timers)
			if value ~= "recast" then
				Timers[7].button:SetAttribute("*unit2", value)
			else
				TotemTimers_EnableEarthShieldRecast()
			end
        end,
        
    EarthShieldMiddleButton =
        function(value, Timers)
			if value ~= "recast" then
				Timers[7].button:SetAttribute("*unit3", value)
			else
				TotemTimers_EnableEarthShieldRecast()
			end
        end,
        
    EarthShieldButton4 =
        function(value, Timers)
			if value ~= "recast" then
				Timers[7].button:SetAttribute("*unit4", value)
			else
				TotemTimers_EnableEarthShieldRecast()
			end
        end,
        
    WeaponEnchants =
        function(value, Timers)
    		for i = 1,2 do
    			if TotemTimers_Spells[value[i]] then
    				Timers[8].button:SetAttribute("spell"..i, TotemTimers_Spells[value[i]])
    			end
    		end
    		for i = 3,4 do
    			if TotemTimers_Spells[value[i]] then
    				Timers[8].button:SetAttribute("shift-spell"..(i-2), TotemTimers_Spells[value[i]])
    				Timers[8].button:SetAttribute("ctrl-spell"..(i-2), TotemTimers_Spells[value[i]])
    				Timers[8].button:SetAttribute("alt-spell"..(i-2), TotemTimers_Spells[value[i]])
    			end
    		end
        end,
        
        
    Order =
        function(value, Timers)
    		for i=1,4 do Timers[i] = getglobal("TotemTimers_Timer"..value[i]).timer end            
    		TotemTimers_OrderTimers()
        end,
        
    OpenMenu =
        function(value, Timers)
            for e=1,4 do
                Timers[e].button:SetAttribute("OpenMenu", value)
                if value == "RightButton" then
                    Timers[e].button:SetAttribute("*macrotext3", "/script TotemTimers.timers["..e.."].earlyExpirationMsgs[1] = nil DestroyTotem("..Timers[e].nr..")")
                    Timers[e].button:SetAttribute("*macrotext2", "")
                else
                    Timers[e].button:SetAttribute("*macrotext3", "")
                    Timers[e].button:SetAttribute("*macrotext2", "/script TotemTimers.timers["..e.."].earlyExpirationMsgs[1] = nil DestroyTotem("..Timers[e].nr..")")
                end
            end
            SetCastButtonsState(value == "always")
        end,
     
    MiniIcons =
        function(value, Timers)
            for e=1,4 do
                if value then
                    Timers[e].button.miniIconFrame:Show()
                else
                    Timers[e].button.miniIconFrame:Hide()
                end
            end
        end,
        
    Lock =
        function(value, Timers)
            for k,v in pairs(Timers) do
                v.locked = value
            end
        end,
        
    Show =
        function(value, Timers)
            if value then
                local available = {}
                for totem, element in pairs(TotemData) do
                    if TotemTimers_Spells[totem] then
                        available[element.element] = true
                    end
                end
                for i=1,4 do
                    if available[Timers[i].nr] then
                        Timers[i]:activate()
                    end
                end
                TotemTimersFrame:Show()
                TotemTimers_OrderTimers()
            else
                for i=1,4 do
                    Timers[i]:deactivate()
                end
                TotemTimersFrame:Hide()
            end
        end,
     
    ProcFlash =
        function(value, Timers)
            for i=1,4 do
                Timers[i].ProcFlash = value
            end
        end,
     
    TimeFont =
        function(value, Timers)
            for _,timer in pairs(Timers) do
                timer:SetFont(value)
            end
        end,
        
    TimerBarTextue =
        function(value, Timers) 
            for _,timer in pairs(Timers) do
                timer:SetBarTexture(value)
            end
        end,
        
    ColorTimerBars =
        function(value, Timers)
            for i=1,4 do
                if value then
                    Timers[i]:SetBarColor(TotemColors[Timers[i].nr][1], TotemColors[Timers[i].nr][2], TotemColors[Timers[i].nr][3])
                else
                    Timers[i]:SetBarColor(0.5,0.5,1.0)
                end
            end
        end,
        
    ShowCooldowns =
        function(value, Timers)
            if TotemTimers_IsSetUp then
                TotemTimers_OrderTimers()
                for i = 1,4 do TotemTimers.TotemEvent(Timers[i].button,"SPELL_UPDATE_COOLDOWN", i) end
            end
        end,
        
    BarBindings =
        function(value, Timers)
            for i=1,4 do
                for j=1,7 do
                    local key = GetBindingKey("TOTEMTIMERSCAST"..i..j)
                    if TotemTimers_Settings.BarBindings and TotemTimers_Settings.OpenMenu ~= "always" then
                        if TotemTimers_Settings.ReverseBarBindings then
                            getglobal("TT_CastButton"..i..j.."HotKey"):SetText(key or tostring(10-j))
                            getglobal("TT_CastButton"..i..j):SetAttribute("binding", tostring(10-j))
                        else
                            getglobal("TT_CastButton"..i..j.."HotKey"):SetText(key or tostring(j))
                            getglobal("TT_CastButton"..i..j):SetAttribute("binding", tostring(j))
                        end
                    else
                        getglobal("TT_CastButton"..i..j.."HotKey"):SetText(key or "")
                        getglobal("TT_CastButton"..i..j):SetAttribute("binding", nil)
                    end
                end
            end
        end,
        
    InMacro1 = function() TotemTimers_UpdateMacro() end,
    InMacro2 = function() TotemTimers_UpdateMacro() end,
    InMacro3 = function() TotemTimers_UpdateMacro() end,
    InMacro4 = function() TotemTimers_UpdateMacro() end,
    
    EnhanceCDs = 
        function(value)
            local _,_,_,_,rank1 = GetTalentInfo(2,28)
            local _,_,_,_,rank2 = GetTalentInfo(2,21)
            local _,_,_,_,rank3 = GetTalentInfo(2,23)
            if value and rank1>0 and rank2>0 and rank3>0 then            
                TotemTimers_ActivateEnhanceCDs() 
            else
                TotemTimers_DeactivateEnhanceCDs()
            end
        end,
        
    EnhanceCDsSize =
        function(value, Timers)
            for i=9,11 do
                Timers[i]:SetScale(value/36)
            end
            TotemTimers_LayoutEnhanceCDs()
        end,
        
    EnhanceCDsTimeHeight = 
         function(value, Timers)
    		for e=9,11 do
    			Timers[e]:SetTimeHeight(value)
    		end        
        end,
        
    EnhanceCDsMaelstromHeight = 
        function(value)
            TotemTimers.maelstrom:SetHeight(value)
            TotemTimers.maelstrom.background:SetHeight(value)
            TotemTimers.maelstrom.icon:SetWidth(value)
            TotemTimers.maelstrom.icon:SetHeight(value)
            local font = TotemTimers.maelstrom.text:GetFont()
    		TotemTimers.maelstrom.text:SetFont(font, value)
        end,
        
    Tooltips =  
        function(value, Timers)
            for i=1,4 do
                Timers[i].button:SetAttribute("tooltip", value)
                for j=1,7 do
                    Timers[i].CastButtons[j]:SetAttribute("tooltip", value)
                end
            end
        end,
        
    TotemTimerBarWidth =
        function(value, Timers)
            for i=1,4 do
                for j=1,Timers[i].numtimers do
                    Timers[i].timerbars[j]:SetWidth(value)
                end
            end
        end,
        
    TrackerTimerBarWidth =
        function(value, Timers)
            for i=5,8 do
                for j=1,Timers[i].numtimers do
                    Timers[i].timerbars[j]:SetWidth(value)
                end
            end
        end,
        
    ActivateHiddenTimers =
        function(value, Timers)
            TotemTimers_ProcessSetting("ShieldTracker")
            TotemTimers_ProcessSetting("EarthShieldTracker")
            TotemTimers_ProcessSetting("WeaponTracker")
        end,
        
    ShowKeybinds =
        function(value, Timers)
            for _,t in pairs(Timers) do
                if value then
                    t.button.hotkey:Show()
                else
                    t.button.hotkey:Hide()
                end
            end
        end,
        
    ShowOmniCCOnEnhanceCDs =
        function(value, Timers)
            for i=9,11 do
                Timers[i].button.cooldown.noCooldownCount = not value
            end
        end,
        
}
SettingsFunctions.ReverseBarBindings = SettingsFunctions.BarBindings

