-- Copyright  2008, 2009 Xianghar  <xian@zron.de>
-- All Rights Reserved.
-- This code is not to be modified or distributed without written permission by the author.
-- Current permissions only include curse.com, wowui.worldofwar.net, wowinterface.com and their respective addon updaters

local totemsqueued = false
local queuedtotems = {}
local inspectQueue = {}
local inspectsqueued = false

local PlayerName = UnitName("player")

local function TotemTimers_ProcessQueue()
    if totemsqueued and not InCombatLockdown() then
        totemsqueued = false
        for i = 1,4 do
            local element = TotemTimers_Settings.Order[i]
            if queuedtotems[element] then
                TotemTimers.timers[i].button:SetAttribute("*spell1", queuedtotems[element])
                queuedtotems[element] = nil
            end                
        end
    end
end


local function SendTalentInfo(mode, recipient)
	local talents = ""
    for i=1,3 do
        for j = 1,GetNumTalents(i) do
            talents = talents..select(5, GetTalentInfo(i,j))
        end
    end
    SendAddonMessage("RAIDTOTEMS", "talents "..talents, mode, recipient)
end

local function SentRTTProtocolVer(mode, recipient)
	SendAddonMessage("RAIDTOTEMS", "protocol 102", mode, recipient)
end

local inRaid = false

function TotemTimers_OnEvent(event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		TotemTimers_SetupGlobals()
		inRaid = GetNumRaidMembers()>0
    elseif event == "LEARNED_SPELL_IN_TAB" then
        TotemTimers_LearnedSpell(select(1,...))
    elseif event == "PLAYER_REGEN_ENABLED" then
        TotemTimers_UpdateMacro()
        TotemTimers_ProcessQueue()
    elseif event == "PLAYER_ALIVE" then
        TotemTimers_Spells.maelstrom = GetTalentInfo(2,28)
        TotemTimersFrame:UnregisterEvent("PLAYER_ALIVE")
        TotemTimers_ProcessSetting("EnhanceCDs")
    elseif event == "CHARACTER_POINTS_CHANGED" then
        local nr = select(1,...)
        if nr > 1 then
            TotemTimers_UnlearnedTalents()
        elseif nr == -1 then
            TotemTimers_LearnedSpell()
        end
	elseif event == "PLAYER_TALENT_UPDATE" then
		TotemTimers_SwitchedTalents()
		if inRaid then
			SendTalentInfo("RAID")
		end
    elseif event == "CHAT_MSG_ADDON" then
        local prefix, msg, _, sender = ...
        if prefix == "RAIDTOTEMS" then
            if msg == "getprotocol" then
                SentRTTProtocolVer("WHISPER", sender)
            elseif msg == "gettalents" then
                SendTalentInfo("WHISPER", sender)
            else
                local args = {strsplit(" ", msg)}
				if args[1] == "es" and args[2] == PlayerName then
					local sn = GetSpellInfo(49284) -- Earth Shield Name, in case shaman is not resto
					TotemTimers_SetEarthShieldRecast(args[3])
					DEFAULT_CHAT_FRAME:AddMessage("TotemTimers: "..sn.." -> "..args[3])
				elseif args[1] == "totems" and args[2] == PlayerName then
                    for i=3,6 do
                        args[i] = tonumber(args[i])
                    end
                    msg = TT_RAIDTOTEMINFO
                    for i=1,4 do
                        local element = i+2--TotemTimers_Settings.Order[i]+2
                        if args[element] and args[element] ~= 0 then
                            local name = GetSpellInfo(args[element])
                            msg = msg..name
							if i<4 then msg = msg..", " end
                            if name then queuedtotems[i] = name end
                            totemsqueued = true
                        end
                    end
					if not InCombatLockdown() then TotemTimers_ProcessQueue() end
                    DEFAULT_CHAT_FRAME:AddMessage(msg)
                end
            end
        end
    elseif event == "RAID_ROSTER_UPDATE" or (event == "PARTY_MEMBERS_CHANGED" and GetNumRaidMembers()==0) then
        TotemTimers_UpdateRaid()
		if not inRaid and GetNumRaidMembers()>0 then
			SentRTTProtocolVer("RAID")
			SendTalentInfo("RAID")
		end
		inRaid = GetNumRaidMembers()>0
	end
end


function TotemTimers_SetupVariables()
	if not TotemTimers_Settings then
		TotemTimers_Settings = {}
	end
	if not TotemTimers_Settings.Version or TotemTimers_Settings.Version ~= 9.0 then
		DEFAULT_CHAT_FRAME:AddMessage("TotemTimers: Pre-9.0 or no saved settings found, loading defaults...")
		TotemTimers_Settings = {}
	end
	TotemTimers_LoadDefaultSettings()
end


function TotemTimers_SetupGlobals()
	if TotemTimers_IsSetUp then
		return
	end
	if select(2,UnitClass("player")) == "SHAMAN" then
		TotemTimers_GetSpells()
		TotemTimers_SetupVariables()
		TotemTimers_CreateTimers()
		TotemTimers_SetCastButtonSpells()
		TotemTimers_CreateTrackers()
        TotemTimers_CreateEnhanceCDs()
		for setting,value in pairs(TotemTimers_Settings) do
			TotemTimers_ProcessSetting(setting)
		end
		TotemTimers_OrderTimers()
		--TotemTimers_OrderTrackers()
		TotemTimersFrame:Show()
        
        TotemTimers_InitSetButtons()
		
		--init weapon tracker, get it to set its textures
		if TotemTimers.timers[8].active then TotemTimers.WeaponEvent(TotemTimers.timers[8].button, "UNIT_INVENTORY_CHANGED") end
		
		--set the slashcommand
		SLASH_TOTEMTIMERS1 = "/totemtimers";
		SLASH_TOTEMTIMERS2 = "/tt";
		SlashCmdList["TOTEMTIMERS"] = TotemTimers_Slash

		TotemTimers_LastGUIPane = TotemTimers_GUI_General

        TotemTimers_InitializeBindings()
        hooksecurefunc("SaveBindings", function() ClearOverrideBindings(TotemTimersFrame) TotemTimers_InitializeBindings() end)    
            
		
        TotemTimersFrame:RegisterEvent("SPELLS_CHANGED")
        TotemTimersFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
        TotemTimersFrame:RegisterEvent("PLAYER_ALIVE")
        TotemTimersFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
        TotemTimersFrame:RegisterEvent("CHARACTER_POINTS_CHANGED")
        TotemTimersFrame:RegisterEvent("CHAT_MSG_ADDON")
        TotemTimersFrame:RegisterEvent("RAID_ROSTER_UPDATE")
        TotemTimersFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
		TotemTimersFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		TotemTimersFrame:RegisterEvent("PLAYER_TALENT_UPDATE")
        TotemTimers_InitButtonFacade()
        TotemTimers_UpdateMacro()
        TotemTimers_UpdateRaid()
	else
		TotemTimersFrame:Hide()
	end
	TotemTimers_IsSetUp = true
    TotemTimersFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function TotemTimers_Slash(msg)
	if InCombatLockdown() then
		DEFAULT_CHAT_FRAME:AddMessage("Can't open TT options in combat.")
		return
	end
	if msg == "i" or msg == "inspect" then
        InterfaceOptionsFrame_OpenToCategory(TotemTimers_GUI_Inspect.name)
    else
        InterfaceOptionsFrame_OpenToCategory(TotemTimers_LastGUIPane.name)
    end
end

function TotemTimers_UpdateMacro()
	if not InCombatLockdown() then
		local _, free = GetNumMacros()
		local nr = GetMacroIndexByName("TT")
		if free==18 and nr==0 then return end
		local sequence = "#showtooltip\n"
        if TotemTimers_Spells[TotemTimers_Icons.TotemicCall] then
            sequence = sequence.."/cast [btn:2] "..TotemTimers_Spells[TotemTimers_Icons.TotemicCall].."\n"
        end
        sequence = sequence .."/castsequence reset=combat/60 "
		for i=1,4 do
			if TotemTimers_Settings["InMacro"..i] then
                local spell = TotemTimers.timers[i].button:GetAttribute("*spell1")
				if spell then
					sequence = sequence .. spell..", "
				end
			end
		end
		sequence = strsub(sequence, 1, strlen(sequence)-2)
		local nr = GetMacroIndexByName("TT")
		if nr == 0 then
			CreateMacro("TT", 1, sequence, 1)
		else
			EditMacro(nr, "TT", 1, sequence, 1)
		end
	else
		TotemTimers_MacroNeedsUpdate = true
	end
end


local text

local function addVar(var, indent)
    if type(var) == "table" then
        text = text.." {|n"
        for k,v in pairs(var) do
            for i=1,indent+4 do text = text.." " end
            text = text..'["'..k..'"] = '
            addVar(v, indent+4)
        end
        for i=1,indent do text = text.." " end
        text = text.."}|n"
    else
        text = text..tostring(var).."|n"
    end
end

function TotemTimers_ShowDebug()
	text = ""
	--[[text = text.."Settings:|n"
	for k,v in pairs(TotemTimers_Settings) do
		text = text..'    ["'..k..'"] = '
        addVar(v, 4)
	end
	text=text.."|n|n"
    text=text.."Available spells:|n"
	for k,v in pairs(TotemTimers_Spells) do
		text = text..'    ["'..k..'"] = '
        addVar(v, 4)
	end  ]]
	text = "EnhanceCDs option: "..tostring(TotemTimers_Settings["EnhanceCDs"]).."|n"
	local name,_,_,_,rank = GetTalentInfo(2,28)
	text = text..tostring(name)..": "..tostring(rank).."|n"
    name,_,_,_,rank = GetTalentInfo(2,21)
	text = text..tostring(name)..": "..tostring(rank).."|n"
    name,_,_,_,rank = GetTalentInfo(2,23)
	text = text..tostring(name)..": "..tostring(rank).."|n"
	text = text.."SS-Timer: "..tostring(TotemTimers.timers[9].active).."|n"
	text = text.."ES-Timer: "..tostring(TotemTimers.timers[10].active).."|n"
	text = text.."LL-Timer: "..tostring(TotemTimers.timers[11].active).."|n"
	text = text.."EC Pos: "
	local a,b,c,d,e = TotemTimers_EnhanceCDsFrame:GetPoint(1)
	if a then
		text=text..a.." "
		if b then text = text..b:GetName() else text = text.."nil " end
		text = text..tostring(c).." "..tostring(d).." "..tostring(e)
	end
	text=text.."|n"
	TotemTimers_Debug:SetText(text)
	TotemTimers_Debug:HighlightText()
end